/**
 * Fixes Overview Tab Module
 * Handles the display and pagination of fixed items
 */
(function($, rank) {
    'use strict';

    // Cache elements
    const $fixesAccordion = $('#rank-fixes-accordion');

    // --- Fixes Overview Tab Logic ---

    // Function to fetch and display initial counts for all categories
    function loadInitialFixCounts() {
        // Check if the accordion exists on the current page view
        if (!$fixesAccordion.length) {
            return; // Exit if not on the Fixes Overview tab
        }

        $.ajax({
            url: rank_ajax_obj.ajax_url,
            type: 'POST',
            data: {
                action: 'rank_get_all_fix_counts',
                nonce: rank_ajax_obj.nonce
            },
            dataType: 'json',
            success: function(response) {
                if (response.success && response.data) {
                    const counts = response.data;
                    // Check if there are any fixes
                    let totalFixes = 0;
                    
                    // Loop through each accordion header
                    $fixesAccordion.find('.rank-accordion-header').each(function() {
                        const $header = $(this);
                        const category = $header.data('category');
                        const issueType = $header.data('issue-type') || category;
                        const $countSpan = $header.find('.rank-accordion-count');
                        let count = 0;

                        // Check if we have a count for this specific issue type
                        if (counts.hasOwnProperty(issueType)) {
                            count = parseInt(counts[issueType]);
                            $countSpan.text(`✅ ${count}`); // Update text with count
                        } else if (counts.hasOwnProperty(category)) {
                            // Fallback to category if issue type not found
                            count = parseInt(counts[category]);
                            $countSpan.text(`✅ ${count}`);
                        } else {
                            $countSpan.text('0'); // Default to 0 if not found in response
                        }
                        
                        totalFixes += count;
                    });
                    
                    // Update the description paragraph based on whether fixes were found
                    const $descriptionParagraph = $('.rank-section p:contains("Expand by clicking")');
                    if (totalFixes === 0) {
                        // No fixes found - show warning message with link to fix problems tab
                        $descriptionParagraph.html('⚠️ No fixes found. <a href="?page=rank-admin&tab=bulk_fix">Go here to fix problems.</a>');
                    } else {
                        // Keep original text if fixes were found
                        if ($descriptionParagraph.text().includes('⚠️')) {
                            $descriptionParagraph.text('Expand by clicking on each to see and edit the changes applied by RANK AI.');
                        }
                    }
                    
                    // Hide the tab content loader after all counts are loaded
                    $('.tab-content-loader').fadeOut(300);
                } else {
                    // Handle error fetching counts
                    $fixesAccordion.find('.rank-accordion-count').text('(?)');
                    console.error('Error fetching fix counts:', response.data || 'Unknown error');
                    
                    // Use helper if available
                    if (rank.helpers && rank.helpers.showNotice) {
                        rank.helpers.showNotice('Could not load fix counts.', 'warning');
                    }
                }
            },
            error: function(jqXHR, textStatus, errorThrown) {
                $fixesAccordion.find('.rank-accordion-count').text('(?)');
                console.error(`AJAX Error fetching fix counts: ${textStatus} - ${errorThrown}`);
                
                // Use helper if available
                if (rank.helpers && rank.helpers.showNotice) {
                    rank.helpers.showNotice(`Error loading fix counts: ${textStatus}`, 'error');
                }
                
                // Hide the tab content loader even if there's an error
                $('.tab-content-loader').fadeOut(300);
            }
        });
    }

    // Accordion Header Click
    $fixesAccordion.on('click', '.rank-accordion-header', function() {
        const $header = $(this);
        const $content = $header.next('.rank-accordion-content');
        const $item = $header.closest('.rank-accordion-item');
        const category = $header.data('category');
        const isOpen = $header.hasClass('rank-accordion-open');
        const $countSpan = $header.find('.rank-accordion-count');
        
        // Ensure the count text starts with a checkmark if it doesn't already
        const countText = $countSpan.text();
        if (!countText.startsWith('✅')) {
            $countSpan.text(`✅ ${countText}`);
        }

        if (isOpen) {
            // Close the accordion
            $content.slideUp(200);
            $header.removeClass('rank-accordion-open');
        } else {
            // Open the accordion
            $content.slideDown(200);
            $header.addClass('rank-accordion-open');

            // Load data if it hasn't been loaded yet for this item
            if (!$item.data('loaded')) {
                loadFixesData(category, 1, $item); // Load page 1
            }
        }
    });

    // Function to load fixes data via AJAX (for a single category when expanded)
    function loadFixesData(category, page, $item) {
        const $content = $item.find('.rank-accordion-content');
        const $tableContainer = $content.find('.rank-accordion-table-container');
        const $paginationContainer = $content.find('.rank-accordion-pagination');
        const $loading = $content.find('.rank-accordion-loading');
        const $noData = $content.find('.rank-accordion-no-data');
        const $header = $item.find('.rank-accordion-header');
        const $countSpan = $header.find('.rank-accordion-count');
        const issueType = $header.data('issue-type') || category;

        $loading.show();
        $noData.hide();
        $tableContainer.find('.rank-fixes-table').remove(); // Clear previous table
        $paginationContainer.empty(); // Clear previous pagination

        $.ajax({
            url: rank_ajax_obj.ajax_url,
            type: 'POST',
            data: {
                action: 'rank_get_fixes_data',
                nonce: rank_ajax_obj.nonce,
                category: category,
                issue_type: issueType,
                page: page
            },
            dataType: 'json',
            success: function(response) {
                if (response.success) {
                    $item.data('loaded', true); // Mark as loaded (at least once)
                    const data = response.data;

                    // Update count in header, always keeping the checkmark
                    // Ensure we always show the checkmark, even for 0 count
                    $countSpan.text(`✅ ${data.total_items}`);

                    if (data.items && data.items.length > 0) {
                        
                        // Build Table
                        displayFixesData(data, $tableContainer, category);

                        // Build Pagination
                        displayPagination(data, $paginationContainer, category, $item);

                    } else {
                        $noData.show();
                        // Ensure we keep the checkmark for 0 items
                        if (data.total_items === 0) {
                            $countSpan.text(`✅ 0`);
                        }
                    }
                } else {
                    $noData.text('Error loading data: ' + (response.data || 'Unknown error')).show();
                    $countSpan.text('(?)'); // Keep (?) on error
                }
            },
            error: function(jqXHR, textStatus, errorThrown) {
                $noData.text(`AJAX Error loading data: ${textStatus}`).show();
                $countSpan.text('(?)'); // Keep (?) on AJAX error
            },
            complete: function() {
                $loading.hide();
            }
        });
    }

    // Function to display fetched fixes data in a table
    function displayFixesData(data, $tableContainer, category) {
        $tableContainer.empty(); // Clear previous table/messages

        if (!data || !data.items || data.items.length === 0) {
            $tableContainer.html('<p class="rank-accordion-no-data">No fixes found for this category.</p>');
            return;
        }

        let tableHtml = '<table class="wp-list-table widefat fixed striped rank-fixes-table">';
        tableHtml += '<thead><tr>';

        // Define headers based on category type
        const isOptionType = data.items[0].is_option; // Check the flag from the first item
        // Check for image alt tags using multiple possible category/issue type values
        const isImageType = category === 'image_alt' || category === 'missing_alt_tags' || (data.items[0].issue_type && data.items[0].issue_type.includes('alt_tag'));

        if (isOptionType) {
            tableHtml += '<th scope="col">Landing Page</th>';
            tableHtml += '<th scope="col">Old URL / Identifier</th>';
            tableHtml += '<th scope="col">New URL / Replacement</th>';
        } else if (isImageType) {
            tableHtml += '<th scope="col">Context (Post & Image)</th>';
            tableHtml += '<th scope="col">Image Preview</th>';
            tableHtml += '<th scope="col">Image URL</th>';
            tableHtml += '<th scope="col">New Alt Text</th>';
        } else { // Standard DB types (Title, Meta Desc)
            tableHtml += '<th scope="col">Context (Post/Page)</th>';
            tableHtml += '<th scope="col">New Value</th>';
        }
        tableHtml += '</tr></thead>';
        tableHtml += '<tbody>';

        data.items.forEach(item => {
            tableHtml += '<tr>';
            if (isOptionType) {
                // For broken links:
                // - context: contains the landing page URL (where the broken link was found)
                // - context_text: contains the old URL (the broken link)
                // - value: contains the new URL (replacement)
                
                // Get the landing page URL
                const landingPageUrl = item.context || '#';
                const displayUrl = landingPageUrl.replace(/^https?:\/\/[^\/]+/, ''); // Show relative URL for cleaner display
                
                // Display the landing page URL with safe link opening
                tableHtml += `<td><a href="javascript:void(0);" onclick="window.open('${landingPageUrl}','_blank');">${displayUrl || 'N/A'}</a></td>`;
                
                // Display the old URL
                tableHtml += `<td>${item.context_text || 'N/A'}</td>`;
                
                // Add edit icon to the replacement value
                tableHtml += `<td>
                    <span class="rank-editable-value">${item.value || 'N/A'}</span>
                    <span class="rank-edit-icon dashicons dashicons-edit" data-type="${category}" data-is-option="true" data-old-url="${encodeURIComponent(item.context_text || '')}" data-issue-type="${item.issue_type || category}"></span>
                </td>`;
            } else if (isImageType) {
                // Context is a link using context_text
                tableHtml += `<td><a href="javascript:void(0);" onclick="window.open('${item.context || '#'}','_blank');">${item.context_text || 'N/A'}</a></td>`;
                
                // Image preview column
                tableHtml += `<td>${item.image_src && item.image_src.startsWith('http') ? `<img src="${item.image_src}" alt="Preview" style="max-width: 50px; max-height: 50px; vertical-align: middle;">` : (item.image_src || 'N/A')}</td>`;
                
                // Image URL column - clickable link to open image in new tab
                // Extract the image ID from context_text which has format "... (Image ID: 12345)"
                let imageId = '';
                const imageIdMatch = item.context_text ? item.context_text.match(/\(Image ID: (\d+)\)/) : null;
                if (imageIdMatch && imageIdMatch[1]) {
                    imageId = imageIdMatch[1];
                }
                
                // Use image_src if available, otherwise construct URL from image ID
                const imageUrl = item.image_src || (imageId ? `/wp-admin/upload.php?item=${imageId}` : '#');
                const displayUrl = imageUrl.startsWith('http') ? imageUrl.split('/').pop() : imageUrl;
                
                tableHtml += `<td><a href="javascript:void(0);" onclick="window.open('${imageUrl}','_blank');">${displayUrl || 'N/A'}</a></td>`;
                
                // New alt text value with edit icon
                tableHtml += `<td>
                    <span class="rank-editable-value">${item.value || 'N/A'}</span>
                    <span class="rank-edit-icon dashicons dashicons-edit" data-post-id="${item.post_id || ''}" data-object-id="${item.object_id || ''}" data-type="${category}" data-issue-type="${item.issue_type || category}"></span>
                </td>`;
            } else { // Standard DB types
                // Context is a link using context_text
                tableHtml += `<td><a href="javascript:void(0);" onclick="window.open('${item.context || '#'}','_blank');">${item.context_text || 'N/A'}</a></td>`;
                // New value (Title or Meta Desc) with edit icon
                tableHtml += `<td>
                    <span class="rank-editable-value">${item.value || 'N/A'}</span>
                    <span class="rank-edit-icon dashicons dashicons-edit" data-post-id="${item.post_id || ''}" data-type="${category}" data-issue-type="${item.issue_type || category}"></span>
                </td>`;
            }
            tableHtml += '</tr>';
        });

        tableHtml += '</tbody></table>';
        $tableContainer.html(tableHtml);
        
        // Add event handlers after the table is added to the DOM
        $tableContainer.on('click', '.rank-edit-icon', handleEditClick);
    }
    
    // Function to handle edit icon clicks
    function handleEditClick() {
        const $icon = $(this);
        const $cell = $icon.parent();
        const $valueSpan = $cell.find('.rank-editable-value');
        const currentValue = $valueSpan.text();
        const isOption = $icon.data('is-option') === true;
        const category = $icon.data('type');
        const oldUrl = isOption ? decodeURIComponent($icon.data('old-url')) : '';
        const postId = $icon.data('post-id');
        const objectId = $icon.data('object-id');
        
        // Hide the current content and icon
        $valueSpan.hide();
        $icon.hide();
        
        // Add edit field and buttons
        $cell.append(`
            <div class="rank-edit-container">
                <input type="text" class="rank-edit-input" value="${currentValue}" style="width: 100%; margin-bottom: 5px;">
                <button class="rank-save-btn button button-small">Save</button>
                <button class="rank-cancel-btn button button-small">Cancel</button>
            </div>
        `);
        
        // Focus the input field
        $cell.find('.rank-edit-input').focus();
        
        // Handle save button click
        $cell.find('.rank-save-btn').on('click', function() {
            const $btn = $(this);
            const $input = $cell.find('.rank-edit-input');
            const newValue = $input.val();
            
            // Show loading indicator
            $btn.text('Saving...').prop('disabled', true);
            $cell.find('.rank-cancel-btn').prop('disabled', true);
            
            // AJAX call to save the new value
            $.ajax({
                url: rank_ajax_obj.ajax_url,
                type: 'POST',
                data: {
                    action: 'rank_update_fix_value',
                    nonce: rank_ajax_obj.nonce,
                    category: category,
                    value: newValue,
                    is_option: isOption,
                    old_url: oldUrl,
                    post_id: postId,
                    object_id: objectId,
                    issue_type: $icon.data('issue-type') || category
                },
                success: function(response) {
                    if (response.success) {
                        // Update the displayed value
                        $valueSpan.text(newValue).show();
                        $icon.show();
                        $cell.find('.rank-edit-container').remove();
                    } else {
                        alert('Error updating value: ' + (response.data || 'Unknown error'));
                        // Restore the edit field
                        $btn.text('Save').prop('disabled', false);
                        $cell.find('.rank-cancel-btn').prop('disabled', false);
                    }
                },
                error: function() {
                    alert('AJAX error occurred while updating value');
                    // Restore the edit field
                    $btn.text('Save').prop('disabled', false);
                    $cell.find('.rank-cancel-btn').prop('disabled', false);
                }
            });
        });
        
        // Handle cancel button click
        $cell.find('.rank-cancel-btn').on('click', function() {
            // Restore the original display
            $valueSpan.show();
            $icon.show();
            $cell.find('.rank-edit-container').remove();
        });
    }

    // Function to display pagination controls
    function displayPagination(data, $paginationContainer, category, $item) {
        $paginationContainer.empty(); // Clear existing controls

        if (data.total_pages <= 1) {
            return; // No pagination needed if 1 page or less
        }

        const $paginationInfo = $('<span class="rank-pagination-info"></span>');
        const $paginationControls = $('<div class="rank-pagination-controls"></div>');

        // Info Text (e.g., "Page 1 of 5 (85 items)")
        const firstItem = (data.current_page - 1) * data.per_page + 1;
        const lastItem = Math.min(data.current_page * data.per_page, data.total_items);
        $paginationInfo.text(
            `${rank_ajax_obj.text_page || 'Page'} ${data.current_page} ${rank_ajax_obj.text_of || 'of'} ${data.total_pages} (${firstItem}-${lastItem} ${rank_ajax_obj.text_of || 'of'} ${data.total_items} ${rank_ajax_obj.text_items || 'items'})`
        );

        // Previous Button
        const $prevButton = $(`<button type="button" class="button button-small rank-prev-page" ${data.current_page === 1 ? 'disabled' : ''}>${rank_ajax_obj.text_prev || 'Previous'}</button>`);

        // Next Button
        const $nextButton = $(`<button type="button" class="button button-small rank-next-page" ${data.current_page === data.total_pages ? 'disabled' : ''}>${rank_ajax_obj.text_next || 'Next'}</button>`);

        // Build pagination controls
        $paginationControls.append($prevButton);
        $paginationControls.append($paginationInfo);
        $paginationControls.append($nextButton);

        // Attach click handlers
        $prevButton.on('click', function() {
            if (data.current_page > 1) {
                loadFixesData(category, data.current_page - 1, $item);
            }
        });

        $nextButton.on('click', function() {
            if (data.current_page < data.total_pages) {
                loadFixesData(category, data.current_page + 1, $item);
            }
        });

        $paginationContainer.append($paginationControls);
    }

    // Initialize - Load fix counts on page load
    $(document).ready(function() {
        loadInitialFixCounts();
    });
    
    // Export functions to rank object for potential use in other modules
    rank.fixesOverview = {
        loadInitialFixCounts: loadInitialFixCounts,
        loadFixesData: loadFixesData,
        displayFixesData: displayFixesData,
        displayPagination: displayPagination
    };

})(jQuery, window.rank = window.rank || {});