/**
 * Health Issues Module
 * Handles fetching and displaying health issues overview and initiating fixes.
 */
(function($, rankLocalizedData) {
    'use strict';
    
    // Module variables
    let $healthIssuesLoading, $healthIssuesContainer, $autoProcessButton;
    
    // Initialize the module
    function init(elements) {
        $healthIssuesLoading = elements.healthIssuesLoading;
        $healthIssuesContainer = elements.healthIssuesContainer;
        $autoProcessButton = elements.autoProcessButton;
        
        // Ensure rank.state is initialized
        rank.state = rank.state || {};
        
        // Check if we're on the health issues tab and load issues
        if ($healthIssuesLoading.length > 0) {
            loadHealthIssuesOverview();
        }
    }
    
    /**
     * Load and display the health issues overview from the server.
     */
    function loadHealthIssuesOverview() {
        // Development logging removed for production

        // Clear processing area when loading health issues overview
        // This ensures it's not visible when directly accessing the tab or reopening the plugin
        if (rank.processing && typeof rank.processing.clearProcessingArea === 'function') {
            rank.processing.clearProcessingArea();
        } else {
            // Fallback if the function isn't available
            if (rank.elements && rank.elements.processingArea) {
                rank.elements.processingArea.hide();
            }
            if (rank.elements && rank.elements.healthIssuesContainer) {
                rank.elements.healthIssuesContainer.show();
            }
        }

        const issuesContainer = $('#rank-health-issues-container');
        const loadingIndicator = $('#rank-health-issues-loading');
        const globalCreditStatusArea = $('#rank-global-credit-status');

        if (!issuesContainer.length || !loadingIndicator.length) {
            console.error('RANK AI: Critical UI elements (issues container or loading indicator) not found.');
            if (loadingIndicator.length) loadingIndicator.hide();
            return;
        }

        // Check if rankLocalizedData and its necessary properties are available
        if (!rankLocalizedData || typeof rankLocalizedData !== 'object' || !rankLocalizedData.ajax_url || !rankLocalizedData.nonce) {
            console.error('RANK AI: rankLocalizedData is missing or incomplete (ajax_url, nonce). Current value:', rankLocalizedData);
            loadingIndicator.hide();
            issuesContainer.html(`<p style="color: red;">Error: Plugin localization data (rankLocalizedData) is missing or incomplete. Cannot load health issues. Please check browser console.</p>`).show();
            return;
        }
        
        // Development logging removed for production

        // Show the container with just the heading, description, and loading indicator
        issuesContainer.show();
        loadingIndicator.show();
        
        // Save the heading, description, and loading indicator before clearing the rest
        const $heading = issuesContainer.find('h2').first();
        const $description = issuesContainer.find('p.description').first();
        const $loadingSection = loadingIndicator.detach(); // Remove but keep reference
        
        // Clear all other content except what we saved
        issuesContainer.children().not($heading).not($description).remove();
        
        // Re-add the heading, description, and loading indicator
        if ($heading.length === 0 || $description.length === 0) {
            // If heading and description weren't found, add them manually
            issuesContainer.empty().append('<h2>Fix health problems automatically</h2>')
                          .append('<p class="description">Get all pages on your website fixed automatically for each health problem found in your Morningscore account.</p>');
        }
        
        // Add the loading indicator back after the heading and description
        issuesContainer.append($loadingSection);

        const request = rank.helpers.trackAjaxRequest($.ajax({
            url: rankLocalizedData.ajax_url,
            type: 'POST',
            data: {
                action: 'rank_get_health_issues_overview',
                nonce: rankLocalizedData.nonce
            },
            success: function(response) {
                loadingIndicator.hide(); // Hide just the loading indicator
                
                // Log timing information if available
                if (response.success && response.data && response.data.debug_timing) {
                    console.log('RANK TIMING: Health issues overview loaded in ' +
                                response.data.debug_timing.execution_time.toFixed(4) + ' seconds');
                    
                    // Log individual issue type timing if available
                    if (response.data.issues) {
                        console.log('RANK TIMING: Breakdown by issue type:');
                        for (const issueType in response.data.issues) {
                            if (response.data.issues[issueType].debug_timing) {
                                console.log('RANK TIMING:   - ' + issueType + ': ' +
                                           response.data.issues[issueType].debug_timing.toFixed(4) + ' seconds');
                            }
                        }
                    }
                }
                
                if (response.success && response.data) {
                    // --- CREDIT HANDLING ---
                    let creditsInfo = { remaining: 0, total: 0 }; // Default structure
                    if (response.data.credits && typeof response.data.credits.limit !== 'undefined' && typeof response.data.credits.usage !== 'undefined') {
                        creditsInfo.total = parseInt(response.data.credits.limit, 10);
                        creditsInfo.remaining = parseInt(response.data.credits.limit, 10) - parseInt(response.data.credits.usage, 10);
                        // Ensure remaining is not negative
                        if (creditsInfo.remaining < 0) {
                            creditsInfo.remaining = 0;
                        }
                    } else {
                        console.warn('Credits data missing or incomplete. Using defaults.');
                    }
                    // --- END CREDIT HANDLING ---

                    const issues = response.data.issues || {};

                    // Update global credit display
                    if (globalCreditStatusArea.length) {
                        globalCreditStatusArea.text(`AI Credits left: ${creditsInfo.remaining} / ${creditsInfo.total}`);
                    }

                    let hasPositiveIssues = false; // Flag to check if any issue type has count > 0
                    let fixedIssueTypesCount = 0; // Counter for issue types with fixed issues
                    let hasBigSiteIssue = false; // Flag to check if any issue type has count > 500
                    const creditsRemaining = creditsInfo.remaining; // Use the calculated remaining credits

                    if (rankLocalizedData && rankLocalizedData.healthCheckCategories && Object.keys(rankLocalizedData.healthCheckCategories).length > 0) {
                        for (const categoryKey in rankLocalizedData.healthCheckCategories) {
                            if (Object.hasOwnProperty.call(rankLocalizedData.healthCheckCategories, categoryKey)) {
                                const categoryLabel = rankLocalizedData.healthCheckCategories[categoryKey];
                                const issueData = issues[categoryKey]; // From AJAX response

                                // Processing category data

                                let issueCount = 0;
                                let needsRecrawl = false; // New variable
                                
                                if (issueData && typeof issueData.count !== 'undefined') {
                                    issueCount = parseInt(issueData.count, 10);
                                    if (isNaN(issueCount)) {
                                        console.error(`Invalid issue count received. Defaulting to 0.`);
                                        issueCount = 0; // Fallback if parseInt fails
                                    }
                                    
                                    // Check for already fixed issues based on sample
                                    needsRecrawl = issueData.needs_recrawl || false;
                                } else {
                                    // This case should be less common if PHP sends all categories
                                    console.warn(`Issue data missing or incomplete. Defaulting count to 0.`);
                                }

                                let boxClass = 'rank-issue-box';
                                let buttonText = '';
                                let buttonDisabled = false;
                                let statusMessage = ''; // Message below the button

                                if (issueCount > 500) {
                                    hasBigSiteIssue = true;
                                }
                                
                                if (issueCount > 0) {
                                    hasPositiveIssues = true;
                                    buttonText = rankLocalizedData.text_fix_issues || 'Fix Issues';
                                    
                                    if (needsRecrawl) {
                                        boxClass += ' rank-issue-box-warning'; // Add warning class
                                        
                                        // Show sample-based message
                                        const sampleSize = issueData.sample_size || 0;
                                        const sampleFixedCount = issueData.sample_fixed_count || 0;
                                        
                                        if (sampleSize > 0) {
                                            statusMessage = `${sampleFixedCount} of ${sampleSize} sampled issues already fixed. Fresh scan recommended.`;
                                        } else {
                                            statusMessage = `Some issues already fixed. Fresh scan recommended.`;
                                        }
                                        
                                        // Increment the counter for issue types with fixed issues
                                        fixedIssueTypesCount++;
                                    }
                                    
                                    if (creditsRemaining <= 0) {
                                        buttonDisabled = true;
                                        statusMessage = rankLocalizedData.text_credits_exhausted || 'AI credits exhausted.';
                                    }
                                } else { // issueCount is 0
                                    boxClass += ' rank-issue-box-good'; // Add class for green border
                                    buttonText = rankLocalizedData.text_all_good || 'All good ✅';
                                    buttonDisabled = true;
                                    // statusMessage remains empty for "All good" cases
                                }

                                let issueBoxHtml = `
                                    <div class="${boxClass}" data-issue-type="${categoryKey}">
                                        <h3>${categoryLabel}</h3>
                                        <p>Problems: <span class="rank-issue-count">${issueCount}</span></p>
                                        <button class="button rank-fix-issues-button" data-issue-type="${categoryKey}" ${buttonDisabled ? 'disabled' : ''}>${buttonText}</button>
                                        <p class="rank-issue-status">${statusMessage}</p>
                                    </div>`;
                                issuesContainer.append(issueBoxHtml);
                            }
                        }

                        // Display an overall message if no issues requiring fixes were found
                        if (!hasPositiveIssues) {
                            if (creditsRemaining > 0) {
                                issuesContainer.append(`<p class="rank-overall-status-message">${rankLocalizedData.text_no_issues_found || 'No health issues found that require fixing. Good job!'}</p>`);
                            } else {
                                // Credits exhausted AND no positive issues
                                issuesContainer.append(`<p class="rank-overall-status-message">${rankLocalizedData.text_no_issues_credits_exhausted || 'No health issues found that require fixing, and AI credits are exhausted.'}</p>`);
                            }
                        }
                        
                        // Add notification button if more than one issue type has already fixed issues
                        if (fixedIssueTypesCount > 1) {
                            issuesContainer.append(`
                                <div class="rank-recrawl-notification">
                                    <p>To get a fresh overview of issues left to fix, <a href="javascript:void(0);" onclick="window.open('https://help.morningscore.io/en/article/how-to-refresh-your-health-data-10o0v7t/','_blank');">scan your website</a></p>
                                </div>
                            `);
                        }
                        
                        // Add warning for big sites if any issue type has more than 500 pages
                        if (hasBigSiteIssue) {
                            issuesContainer.append(`
                                <div style="float: left; clear: both; margin-top: 20px; padding: 15px; background-color: #fff8e5; border-left: 4px solid #ffb900; box-shadow: 0 1px 1px rgba(0,0,0,.04);">
                                    <p>⚠️ This plugin is not optimal for bigger sites yet. It might freeze. Contact us on chat for advice 😊</p>
                                </div>
                            `);
                        }
                        
                        issuesContainer.show();

                    } else {
                        // No healthCheckCategories configured or it's empty
                        issuesContainer.html(`<p>${rankLocalizedData.text_config_missing || 'Error: Health check categories configuration is missing.'}</p>`).show();
                    }
                } else {
                    // AJAX call was successful but response.success was false or no data
                    const errorMessage = response.data && response.data.message ? response.data.message : (response.data || 'Unknown error');
                    issuesContainer.html(`<p>${rankLocalizedData.text_error_loading_issues || 'Error loading health issues:'} ${errorMessage}</p>`).show();
                }
            },
            error: function(jqXHR, textStatus, errorThrown) {
                loadingIndicator.hide(); // Hide just the loading indicator
                
                // If the request was aborted, don't show an error message
                if (textStatus === 'abort') {
                    return; // Exit silently without showing error message
                }
                
                console.error('AJAX Error in loadHealthIssuesOverview:', {
                    textStatus: textStatus,
                    errorThrown: errorThrown,
                    status: jqXHR.status,
                    readyState: jqXHR.readyState,
                    responseText: jqXHR.responseText,
                    jqXHR: jqXHR
                });

                // Clear the issues container completely to remove any previous content
                issuesContainer.empty();
                
                // Re-add the heading and description
                issuesContainer.append('<h2>Fix health problems automatically</h2>')
                              .append('<p class="description">Get all pages on your website fixed automatically for each health problem found in your Morningscore account.</p>');
                
                // Only show error message for non-abort errors
                let detailedError = `Connection issue: ${textStatus}.`;
                if (errorThrown) {
                    detailedError += ` errorThrown: ${errorThrown}.`;
                }
                if (jqXHR.status !== undefined) {
                     detailedError += ` HTTP Code: ${jqXHR.status}.`;
                }
                
                // Check if this is a domain not found error
                let responseData = null;
                try {
                    responseData = JSON.parse(jqXHR.responseText);
                } catch (e) {
                    // Not valid JSON, continue with default error message
                }
                
                if (responseData &&
                    responseData.success === false &&
                    responseData.data &&
                    responseData.data.message &&
                    responseData.data.message.includes('domain was not found in your Morningscore account')) {
                    // Display specific message for domain not found error
                    issuesContainer.append(`<div class="rank-connection-error"><p>⚠️ This website was not found in your Morningscore account. Please write us in our chat if you need help 💬</p></div>`);
                } else {
                    // Display generic error message for other errors
                    issuesContainer.append(`<div class="rank-connection-error"><p>⚠️ Could not connect to Morningscore. ${detailedError} Please write us in our chat 💬</p></div>`);
                }
                
                // Make sure the container is visible
                issuesContainer.show();
            }
        }));
    }
    
    // Prepare health issues for processing
    function prepareHealthIssues(issueType) {
        // Ensure rankLocalizedData is available for this function too
        if (!rankLocalizedData || !rankLocalizedData.ajax_url || !rankLocalizedData.nonce || !rankLocalizedData.status_preparing) {
            console.error('rankLocalizedData is missing or incomplete for prepareHealthIssues.');
            rank.helpers.showNotice('Error: Plugin configuration data is missing. Cannot prepare issues.', 'error');
            return;
        }

        // Clear previous state
        rank.helpers.clearNotices();
        rank.elements.logOutput.empty();
        rank.helpers.updateProgressBar(0);
        rank.elements.counts.text('');
        $('#rank-approval-area').empty().hide(); // Keep hidden until Manual button is clicked
        
        // Remove any existing credit warning messages
        $('.rank-credits-warning').remove();
        
        // Reset cancel button to normal state
        if (rank.elements.cancelBtn) {
            rank.elements.cancelBtn.removeClass('cancelling').text('Cancel').prop('disabled', false);
        }
        
        // Hide the health issues container and refresh button
        rank.elements.healthIssuesContainer.hide();
        $('.rank-refresh-button').hide();
        rank.elements.processingArea.show();
        rank.elements.statusMessage.html('<span class="spinner is-active" style="float: none; margin-right: 5px;"></span>' + (rankLocalizedData.status_preparing || 'Preparing issues...')); // Use rankLocalizedData with spinner
        
        // Hide manual and automatic buttons during preparation
        if (rank.elements.startButton) {
            rank.elements.startButton.hide();
        }
        if (rank.elements.autoProcessButton) {
            rank.elements.autoProcessButton.hide();
        }
        
        // Store the issue type
        rank.elements.selectedIssueType.val(issueType);
        
        const request = rank.helpers.trackAjaxRequest($.ajax({
            url: rankLocalizedData.ajax_url, // Use rankLocalizedData
            type: 'POST',
            data: {
                action: 'rank_prepare_health_issues',
                nonce: rankLocalizedData.nonce, // Use rankLocalizedData
                issue_type: issueType
            },
            dataType: 'json',
            success: function(response) {
                if (response.success) {
                    const batchIdValue = response.data.batchId;
                    const itemCount = response.data.count;
                    
                    // If itemCount is 0, it means no issues were prepared (e.g., already processed or API returned none)
                    if (itemCount === 0 && response.data.message) {
                         rank.helpers.showNotice(response.data.message, 'info');
                         rank.elements.statusMessage.text(response.data.message);
                         rank.elements.healthIssuesContainer.show();
                         $('.rank-refresh-button').show(); // Show the refresh button again
                         rank.elements.processingArea.hide();
                         return;
                    }
                    if (itemCount === 0 && !response.data.message) {
                         rank.helpers.showNotice('No issues found to prepare for this category.', 'info');
                         rank.elements.statusMessage.text('No issues to prepare.');
                         rank.elements.healthIssuesContainer.show();
                         $('.rank-refresh-button').show(); // Show the refresh button again
                         rank.elements.processingArea.hide();
                         return;
                    }
                    
                    rank.state.batchId = batchIdValue;
                    rank.state.totalCount = itemCount;
                    rank.state.autoApproveFlag = false;
                    rank.state.issueType = issueType;
                    
                    localStorage.setItem('rank_batch_id', batchIdValue);
                    localStorage.setItem('rank_auto_approve', '0');
                    
                    // Update UI
                    rank.elements.batchIdInput.val(batchIdValue);
                    // rank.elements.startButton // This button will no longer be shown
                    //     .show()
                    //     .prop('disabled', false)
                    //     .text(rank_ajax_obj.start_fixing_text || 'Start Fixing');
                    
                    rank.elements.autoProcessButton.show();
                    
                    rank.elements.statusMessage.text(
                        // If you have a specific status message for "starting processing" in rankLocalizedData, use it.
                        // Otherwise, consider adding one or using a generic message.
                        // For now, let's assume a general message or that it's handled by subsequent functions.
                        rankLocalizedData.status_starting_processing || 'Starting to process issues...' 
                    );
                    
                    rank.helpers.addLogMessage(`Preparation complete. Found ${itemCount} problems. Starting automatic fixing... You can leave this tab but don't close it!`);
                    
                    // Get the health issue name based on the current issue type
                    let healthIssueName = '';
                    if (issueType && rankLocalizedData && rankLocalizedData.healthCheckCategories && rankLocalizedData.healthCheckCategories[issueType]) {
                        healthIssueName = rankLocalizedData.healthCheckCategories[issueType] + ': ';
                    }
                    
                    // Display the issue count
                    rank.elements.counts.text(`${healthIssueName}${itemCount} problems`);
                    
                    // Show buttons for user to choose how to proceed
                    rank.elements.statusMessage.text(rankLocalizedData.status_starting_processing || 'Starting to process issues...');
                    rank.helpers.addLogMessage('Preparation complete. User to choose processing mode.');
                    rank.elements.statusMessage.text(rank_ajax_obj.status_prepared || 'Ready. Choose how to proceed:');

                    if (rank.elements.startButton) {
                        rank.elements.startButton.text(rank_ajax_obj.text_process_manually || 'Manual').show();
                    }
                    if (rank.elements.autoProcessButton) {
                        // Ensure autoProcessButton is in its initial "setup" state
                        rank.elements.autoProcessButton.text(rank_ajax_obj.text_auto_process || 'Automatic').data('action', 'setup').show();
                    }
                    if (rank.elements.progressBarContainer) {
                        rank.elements.progressBarContainer.hide();
                    }
                    if (rank.elements.approvalArea) {
                        rank.elements.approvalArea.empty().hide(); // Keep hidden until Manual button is clicked
                    }
                    // DO NOT call fetchNextRow() here. User will choose a mode.
                    
                    // Check if user has enough credits to fix all issues
                    if (rank.state.totalCount > 0) {
                        // Get the current credits remaining from the global credit status area
                        const $globalCreditStatusArea = $('#rank-global-credit-status');
                        let creditsRemaining = 0;
                        
                        if ($globalCreditStatusArea.length) {
                            const creditText = $globalCreditStatusArea.text();
                            const creditMatch = creditText.match(/AI Credits left: (\d+)/);
                            if (creditMatch && creditMatch[1]) {
                                creditsRemaining = parseInt(creditMatch[1], 10);
                            }
                        }
                        
                        // If credits remaining is less than total issues to fix, show warning
                        if (creditsRemaining < rank.state.totalCount) {
                            // Create warning message
                            const $warningBox = $('<div>', {
                                class: 'rank-credits-warning',
                                style: 'background-color: #fff8e5; border-left: 4px solid #ffb900; padding: 10px; margin-top: 10px; margin-bottom: 10px;'
                            }).html(`Fixing all these problems takes ${rank.state.totalCount} credits but you only have ${creditsRemaining} available. You can still continue and fix things until you run out.`);
                            
                            // Insert warning after the buttons
                            if (rank.elements.autoProcessButton) {
                                rank.elements.autoProcessButton.after($warningBox);
                            }
                        }
                    }
                } else {
                    rank.helpers.showNotice((response.data || 'Error preparing issues.'), 'error');
                    rank.helpers.addLogMessage('Error during preparation: ' + (response.data || 'Unknown error'));
                    
                    // Show the issues container and refresh button again
                    rank.elements.healthIssuesContainer.show();
                    $('.rank-refresh-button').show();
                    rank.elements.processingArea.hide();
                }
            },
            error: function(jqXHR, textStatus, errorThrown) {
                // Ensure rank.helpers and rank.elements are available if this part of the code is reached
                if (rank && rank.helpers && rank.elements) {
                    rank.helpers.handleAjaxError({ jqXHR, textStatus, errorThrown }, 'health issues preparation');
                    rank.elements.healthIssuesContainer.show();
                    $('.rank-refresh-button').show();
                    rank.elements.processingArea.hide();
                } else {
                    console.error('AJAX Error in prepareHealthIssues and rank.helpers/elements not available.');
                    // Fallback display if rank.helpers is not available
                    $('#rank-notice-area').html('<div class="notice notice-error"><p>Error preparing health issues. Critical UI components not found.</p></div>');
                }
            }
        }));
    }
    
    // Expose public methods
    rank.healthIssues = {
        init: init,
        loadHealthIssuesOverview: loadHealthIssuesOverview,
        prepareHealthIssues: prepareHealthIssues
    };
    
})(jQuery, window.rankLocalizedData); 