/**
 * Processing Module
 * Handles task scheduling and processing
 */
(function($, rank) {
    'use strict';

    if (typeof rank_ajax_obj === 'undefined' || !rank_ajax_obj) {
        console.error('RANK Critical Error: rank_ajax_obj is not defined in processing.js. Script will halt.');
        // Optionally, try to alert or display a message on the page if console is not open
        // alert('RANK Plugin Critical Error: Essential configuration (rank_ajax_obj) missing. Please contact support or check browser console.');
        return; // Stop execution of this IIFE if critical object is missing
    }
    
    // Schedule all tasks for processing
    function scheduleAllTasks() {
        rank.elements.startButton.prop('disabled', true).hide();
        rank.elements.statusMessage.text(rank_ajax_obj.status_scheduling || 'Scheduling tasks...');
        rank.helpers.addLogMessage('Scheduling all tasks for processing...');
        
        const request = rank.helpers.trackAjaxRequest($.ajax({
            url: rank_ajax_obj.ajax_url,
            type: 'POST',
            data: {
                action: 'rank_schedule_tasks',
                nonce: rank_ajax_obj.nonce,
                batch_id: rank.state.batchId
            },
            dataType: 'json',
            success: function(response) {
                if (response.success) {
                    rank.helpers.addLogMessage('All tasks scheduled successfully: ' + response.data.message);
                    rank.helpers.showNotice('Tasks scheduled. Processing will begin shortly.', 'success');
                    rank.elements.statusMessage.text('Monitoring progress...');
                    
                    // Start polling for status updates
                    startStatusPolling();
                } else {
                    rank.helpers.showNotice((rank_ajax_obj.error_scheduling || 'Error scheduling tasks.') + ' ' + (response.data || ''), 'error');
                    rank.helpers.addLogMessage('Error scheduling tasks: ' + (response.data || 'Unknown error'));
                    rank.elements.startButton.prop('disabled', false).show();
                }
            },
            error: function(jqXHR, textStatus, errorThrown) {
                // If the request was aborted, handle silently
                if (textStatus === 'abort') {
                    return; // Exit silently without showing error message
                }
                
                rank.helpers.showNotice((rank_ajax_obj.error_scheduling || 'Error scheduling tasks.') + ` (AJAX Error: ${textStatus})`, 'error');
                rank.helpers.addLogMessage(`AJAX Error during scheduling: ${textStatus} - ${errorThrown}`);
                rank.elements.startButton.prop('disabled', false).show();
            }
        }));
    }
    
    // Start polling for status updates
    function startStatusPolling() {
        // Implementation...
    }
    
    // Reset all processing state
    function resetProcessingState() {
        // Reset state variables
        rank.state.isAutoProcessingActive = false;
        rank.state.autoApproveFlag = false;
        rank.state.autoProcessActionCompleted = false;
        rank.state.processedCount = 0;
        
        // Reset UI elements
        if (rank.elements.progressBarContainer) {
            rank.elements.progressBarContainer.hide();
        }
        
        if (rank.elements.approvalArea) {
            rank.elements.approvalArea.empty().hide();
        }
        
        // Hide processing area and show health issues
        if (rank.elements.processingArea) {
            rank.elements.processingArea.hide();
        }
        
        if (rank.elements.healthIssuesContainer) {
            rank.elements.healthIssuesContainer.show();
        }
        
        // Reset buttons
        if (rank.elements.startButton) {
            rank.elements.startButton.prop('disabled', false).show();
        }
        
        if (rank.elements.autoProcessButton) {
            rank.elements.autoProcessButton
                .removeClass('button-secondary')
                .addClass('button-primary')
                .show();
        }
        
        // Hide spinner and log container
        $('#rank-log-spinner').removeClass('is-active').hide();
        $('.rank-log-container').hide();
        
        // Reset cancel button to normal state
        if (rank.elements.cancelBtn) {
            rank.elements.cancelBtn.removeClass('cancelling').text('Cancel').prop('disabled', false);
        }
        
        // Clear localStorage flags
        localStorage.removeItem('rank_review_started');
        localStorage.setItem('rank_auto_approve', '0');
        
        // Add log message
        rank.helpers.addLogMessage('Processing state has been completely reset.');
    }
    
    // Start manual review process
    function startManualReview() {
        rank.state.processedCount = 0;
        rank.elements.startButton.hide();
        rank.elements.autoProcessButton.hide();
        
        // Show and initialize progress bar with animation
        rank.elements.progressBarContainer.show();
        rank.helpers.updateProgressBar(0);
        
        // Get the health issue name based on the current issue type
        let healthIssueName = '';
        if (rank.state.issueType && window.rankLocalizedData && window.rankLocalizedData.healthCheckCategories && window.rankLocalizedData.healthCheckCategories[rank.state.issueType]) {
            healthIssueName = window.rankLocalizedData.healthCheckCategories[rank.state.issueType] + ': ';
        }
        
        // Update status message to show review progress
        rank.elements.statusMessage.text(`Review: 1 of ${rank.state.totalCount}`);
        
        // Update counts display with health issue name
        rank.elements.counts.text(`${healthIssueName}${rank_ajax_obj.text_decide_label || 'Processed'}: 1 of ${rank.state.totalCount}`);
        
        // Show loading placeholder immediately before fetching the first item
        if (rank.elements.approvalArea) {
            rank.elements.approvalArea.empty().append(`
                <div class="rank-approval-row">
                    <div class="rank-loading-placeholder">Loading... 5-15 seconds</div>
                </div>
            `).show();
        }
        
        // Hide the spinner for manual processing (we have the loading placeholder now)
        $('#rank-log-spinner').removeClass('is-active').hide();
        
        // Get the first item
        fetchNextRow();
    }
    
    // Helper to extract solution for auto-approval
    function _getSolutionForAutoApproval(solutionData) {
        // Handle array with object inside (most common case from logs)
        if (Array.isArray(solutionData) && solutionData.length > 0) {
            const firstItem = solutionData[0];
            if (typeof firstItem === 'string') {
                return firstItem;
            } else if (firstItem && typeof firstItem === 'object') {
                // If it has solution_value property, use that
                if (firstItem.solution_value) {
                    return firstItem.solution_value;
                }
                // Otherwise try to stringify it
                return JSON.stringify(firstItem);
            }
            return String(firstItem);
        }
        // Handle string
        else if (typeof solutionData === 'string') {
            return solutionData;
        }
        // Handle object
        else if (solutionData && typeof solutionData === 'object') {
            if (solutionData.solution_value) {
                return solutionData.solution_value;
            }
            return JSON.stringify(solutionData);
        }
        
        return ''; // Default to empty if no suitable solution
    }

    // Helper to extract URLs from issue_data
    function _extractUrlsFromIssueData(issue_data, api_issue_type) {
        let page_url = null;
        let item_url = null;
        if (issue_data) {
            page_url = issue_data.page_url || null;
            if (api_issue_type === 'missing_alt_tags' && issue_data.image_url) {
                item_url = issue_data.image_url;
            }
            // Add other conditions for item_url if different issue types have specific item URLs in issue_data
        }
        return { page_url, item_url };
    }
    
    // Helper to update counts and progress bar
    function _updateCountsAndProgressOnFetch(responseData, isManualListItem) {
        rank.state.processedCount = responseData.decisions ? Object.keys(responseData.decisions).length : 0;
        rank.state.totalCount = responseData.totalCount || rank.state.totalCount;
        
        const itemsProcessedOrListed = rank.state.processedCount; // For auto-approve, this is accurate. For list, it's items decided.
                                                              // For progress of listing, we might need another counter if totalCount is large.
                                                              // For now, progress bar reflects decided items.
        
        const pct = rank.state.totalCount > 0 ? Math.round((itemsProcessedOrListed / rank.state.totalCount) * 100) : 0;
        rank.helpers.updateProgressBar(pct);

        // Adjust display count for current item if not in a batch listing mode where all are shown
        // or if it's not auto-approving (where user doesn't see "item X of Y")
        let displayCount = itemsProcessedOrListed;
        if (!rank.state.isAutoProcessingActive || (rank.state.isAutoProcessingActive && !rank.state.autoApproveFlag && !isManualListItem)) {
             // If it's the standard manual one-by-one, or the first item of a batch manual list before loop starts proper
            displayCount = itemsProcessedOrListed + 1;
        }
         if (rank.state.isAutoProcessingActive && rank.state.autoApproveFlag) {
            // For auto-approval, show actual processed count
            displayCount = itemsProcessedOrListed;
        }


        // Get the health issue name based on the current issue type
        let healthIssueName = '';
        if (rank.state.issueType && window.rankLocalizedData && window.rankLocalizedData.healthCheckCategories && window.rankLocalizedData.healthCheckCategories[rank.state.issueType]) {
            healthIssueName = window.rankLocalizedData.healthCheckCategories[rank.state.issueType] + ': ';
        }

        // Update the counts display
        rank.elements.counts.text(`${healthIssueName}${rank_ajax_obj.text_decide_label || 'Processed'}: ${displayCount} of ${rank.state.totalCount}`);
        
        // Update the status message to show progress
        
        if (rank.state.isAutoProcessingActive && rank.state.autoApproveFlag) {
            // For auto-approval mode
            rank.elements.statusMessage.text(`${healthIssueName}Auto-processing: ${displayCount} of ${rank.state.totalCount}`);
        } else {
            // For manual review mode
            rank.elements.statusMessage.text(`${healthIssueName}Review: ${displayCount} of ${rank.state.totalCount}`);
        }
    }

    // Helper to handle batch completion
    function _handleBatchCompletionOnFetch(responseData) {
        // Log state before completion for debugging
        console.log('RANK DEBUG - State before completion:', {
            isAutoProcessingActive: rank.state.isAutoProcessingActive,
            autoApproveFlag: rank.state.autoApproveFlag,
            batchId: rank.state.batchId,
            reviewStarted: localStorage.getItem('rank_review_started')
        });

        const completionMessage = rank.state.isAutoProcessingActive && rank.state.autoApproveFlag ?
                                 (rank_ajax_obj.status_auto_approve_complete || 'Auto-approval Complete. All items processed.') :
                                 (rank_ajax_obj.status_complete || 'Processing Complete. All items reviewed.');
        rank.helpers.showNotice(completionMessage, 'success');
        rank.helpers.addLogMessage('Automatic processing completed for all pages ✅');
        
        rank.elements.statusMessage.text(completionMessage);
        
        // Get the health issue name based on the current issue type
        let healthIssueName = '';
        if (rank.state.issueType && window.rankLocalizedData && window.rankLocalizedData.healthCheckCategories && window.rankLocalizedData.healthCheckCategories[rank.state.issueType]) {
            healthIssueName = window.rankLocalizedData.healthCheckCategories[rank.state.issueType] + ': ';
        }
        
        // Update the counts display to show "Processing Complete!" instead of "Review: X of Y"
        rank.elements.counts.text(`${healthIssueName}Processing Complete!`);
        
        // Add completion video below the headline
        if (rank.elements.counts && rank.elements.counts.length) {
            // Remove any existing video first (in case of multiple completions)
            $('#rank-completion-video').remove();
            
            // Create and append the video element
            // Use the plugin URL provided by WordPress
            // Since we're always in the admin area, we can simplify this code
            const pluginDirName = 'rank-ai-by-morningscore'; // The actual plugin directory name
            
            // Use WordPress-provided URL if available, otherwise use a simple admin fallback
            let pluginPath;
            
            if (rank_ajax_obj.plugin_url) {
                // Handle the case where plugin_url might already have a trailing slash
                pluginPath = rank_ajax_obj.plugin_url.endsWith('/')
                    ? rank_ajax_obj.plugin_url + 'admin/done-guide.mp4'
                    : rank_ajax_obj.plugin_url + '/admin/done-guide.mp4';
            } else {
                // Simple fallback for admin area
                pluginPath = '../wp-content/plugins/' + pluginDirName + '/admin/done-guide.mp4';
            }
                
            const $video = $('<video>', {
                id: 'rank-completion-video',
                src: pluginPath,
                autoplay: true,
                muted: true,
                playsinline: true,
                style: 'display: block; max-width: 60%; margin-top: 10px;'
            });
            
            // Disable controls and looping
            $video.prop('controls', false);
            $video.prop('loop', false);
            
            // Create "See Fixes" button
            const $seeFixesBtn = $('<button>', {
                id: 'rank-see-fixes-button',
                text: 'See Fixes',
                class: 'button button-primary',
                style: 'margin-top: 15px; margin-bottom: 20px;'
            });
            
            // Add click handler to the button
            $seeFixesBtn.on('click', function() {
                // Navigate to the fixes overview page
                window.location.href = 'admin.php?page=rank-admin&tab=fixes_overview';
            });
            
            // Create a container for the video and button
            const $completionContainer = $('<div>', {
                id: 'rank-completion-container',
                style: 'text-align: center;'
            });
            
            // Add the video and button to the container
            $completionContainer.append($video);
            $completionContainer.append($seeFixesBtn);
            
            // Append the container after the counts element
            rank.elements.counts.after($completionContainer);
        }
        
        if (rank.elements.cancelBtn) rank.elements.cancelBtn.hide(); // This might be autoProcessButton now
        if (rank.elements.autoProcessButton) rank.elements.autoProcessButton.hide(); // Hide stop button

        // Create and show a "Done" button instead of automatically closing the processing view
        if (rank.elements.completeClearLogBtn) {
            rank.elements.completeClearLogBtn.text(rank_ajax_obj.text_done || 'Done').show();
        }
        
        if (rank.elements.approvalArea) {
            // Keep approval area visible but empty when batch processing is complete
            rank.elements.approvalArea.empty();
        }
        
        // Keep processing area visible instead of hiding it and showing health issues
        // if (rank.elements.healthIssuesContainer) rank.elements.healthIssuesContainer.show();
        // if (rank.elements.processingArea) rank.elements.processingArea.hide();
        
        // Reset all state variables that affect isProcessingActive()
        rank.state.isAutoProcessingActive = false; // Reset mode
        
        // Clear localStorage flag for both manual and automatic modes
        localStorage.removeItem('rank_review_started');
        
        // Log state after completion for debugging
        console.log('RANK DEBUG - State after completion:', {
            isAutoProcessingActive: rank.state.isAutoProcessingActive,
            autoApproveFlag: rank.state.autoApproveFlag,
            batchId: rank.state.batchId,
            reviewStarted: localStorage.getItem('rank_review_started')
        });
        
        // Hide the spinner when processing is complete
        $('#rank-log-spinner').removeClass('is-active').hide();
        
    }

    // Helper to handle fetch errors
    function _handleFetchErrorOrUnexpected(response) {
        console.warn('Request successful but data indicates an issue or unexpected structure.');
        const errorMessage = (response && response.data && response.data.message) ? response.data.message : (rank_ajax_obj.error_fetching_item || 'Error fetching next item or unexpected response.');
        rank.helpers.showNotice(errorMessage, 'error');
        rank.helpers.addLogMessage('Error fetching next item: ' + errorMessage);
        if (rank.elements.healthIssuesContainer) rank.elements.healthIssuesContainer.show();
        if (rank.elements.processingArea) rank.elements.processingArea.hide();
        if (rank.elements.autoProcessButton) { // Show auto process button again
            rank.elements.autoProcessButton.show();
        }
        
        // Hide the spinner when there's an error
        $('#rank-log-spinner').removeClass('is-active').hide();
        
        rank.state.isAutoProcessingActive = false; // Reset mode
    }


    // Fetch the next row for review (modified to be a central loop)
    function fetchNextRow() {
        // If auto-processing mode is explicitly OFF, and no item is currently displayed for action,
        // then fetchNextRow should only proceed if it's the very first call after "Fix Issues"
        // or if explicitly called by an action like approve/skip.
        // The `rank.state.isAutoProcessingActive` is initialized to false.
        // It only becomes true when "Automatic" button is clicked.
        // It becomes false again when the "Cancel" button is clicked.

        // The initial call from prepareHealthIssues will have isAutoProcessingActive = false.
        // Subsequent calls from approveItem/skipItem will also have isAutoProcessingActive = false (unless user clicked Automatic).
        // If user clicks "Cancel", isAutoProcessingActive becomes false. If an item was auto-approved and then canceled,
        // fetchNextRow might be called one last time by approveItem.
        
        // This condition is to prevent an infinite loop if "Cancel" is clicked
        // during a batch mode, and then fetchNextRow is called by a completing approveItem.
        if (rank.state.isAutoProcessingActive === false &&
            rank.state.autoProcessActionCompleted === true && // A flag to indicate we just finished an auto-process action
            !$('#rank-approval-area .rank-approval-row').length // And no new manual item is up
           ) {
            rank.state.autoProcessActionCompleted = false; // Reset flag
            return;
        }
        // Ensure autoProcessActionCompleted is initialized if not present
        if (typeof rank.state.autoProcessActionCompleted === 'undefined') {
            rank.state.autoProcessActionCompleted = false;
        }


        if (!rank.state.batchId) {
            console.error('Aborting, batchId is missing.');
            rank.helpers.showNotice('Error: Batch ID is missing, cannot fetch next item.', 'error');
            if (rank.elements.healthIssuesContainer) rank.elements.healthIssuesContainer.show();
            if (rank.elements.processingArea) rank.elements.processingArea.hide();
            return;
        }

        const request = rank.helpers.trackAjaxRequest($.ajax({
            url: rank_ajax_obj.ajax_url,
            type: 'POST',
            data: {
                action: 'rank_get_batch_state',
                nonce: rank_ajax_obj.nonce,
                batchId: rank.state.batchId
            },
            dataType: 'json',
            success: function(response) {

                // Process UI logs from the response
                if (response && response.success && response.data && response.data.ui_logs && Array.isArray(response.data.ui_logs)) {
                    // Check for URL validation logs
                    response.data.ui_logs.forEach(logMessage => {
                        if (typeof logMessage === 'string' && logMessage.includes('Skipping URL as it does not exist on our website:')) {
                            // Extract the URL from the log message
                            const urlMatch = logMessage.match(/Skipping URL as it does not exist on our website: (.+)/);
                            const url = urlMatch && urlMatch[1] ? urlMatch[1] : 'unknown URL';
                            
                            // Add a formatted log message to the UI
                            rank.helpers.addLogMessage(`🐞 URL to fix does not exist on our website: ${url}`);
                        } else if (typeof logMessage === 'string') {
                            // Add other log messages to the UI
                            rank.helpers.addLogMessage(logMessage);
                        }
                    });
                }

                if (response && response.success && response.data) {
                    if (response.data.nextRow) {
                        const nextRowData = response.data.nextRow;
                        
                        // Ensure progress bar is visible during processing
                        if (rank.elements.progressBarContainer && !rank.elements.progressBarContainer.is(':visible')) {
                            rank.elements.progressBarContainer.show();
                        }
                        
                        // Update credits counter if available in the response
                        if (response.data.credits) {
                            const $globalCreditStatusArea = $('#rank-global-credit-status');
                            if ($globalCreditStatusArea.length) {
                                $globalCreditStatusArea.text(`AI Credits left: ${response.data.credits.remaining} / ${response.data.credits.total}`);
                            }
                        }
                        
                        _updateCountsAndProgressOnFetch(response.data, false); // Update counts first

                        if (rank.state.isAutoProcessingActive && rank.state.autoApproveFlag) {
                            // Batch Auto-Approval Mode
                            const solutionToApprove = _getSolutionForAutoApproval(nextRowData.solution);
                            const { page_url, item_url } = _extractUrlsFromIssueData(nextRowData.issue_data, rank.state.issueType);
                            
                            // Check if still in auto-processing mode before approving (user might have clicked stop)
                            if (!rank.state.isAutoProcessingActive) {
                                rank.helpers.addLogMessage('Auto-processing stopped. Item not auto-approved.');
                                // Potentially show the item for manual review or reset UI
                                rank.approval.renderApprovalRow(nextRowData); // Show the item that wasn't auto-approved
                                return;
                            }
                            
                            // Store the current row data for URL extraction in handleDecisionResponse
                            if (rank.state) {
                                rank.state.currentRowData = nextRowData;
                            }
                            
                            rank.approval.approveItem(nextRowData.rowKey, solutionToApprove, page_url, item_url, rank.state.issueType);
                            // approveItem will call fetchNextRow again, continuing the loop.
                        } else {
                            // Standard Manual One-by-One Mode (or first item before auto-process mode fully kicks in)
                            rank.approval.renderApprovalRow(nextRowData);
                        }
                    } else {
                        // No nextRow: Batch complete
                        _updateCountsAndProgressOnFetch(response.data, false); // Ensure counts are final
                        _handleBatchCompletionOnFetch(response.data);
                    }
                } else {
                    // Error or unexpected structure
                    _handleFetchErrorOrUnexpected(response);
                }
            },
            error: function(jqXHR, textStatus, errorThrown) {
                // If the request was aborted, handle silently
                if (textStatus === 'abort') {
                    return; // Exit silently without showing error message
                }
                
                console.error('AJAX error fetching next item.');
                rank.helpers.showNotice(`${rank_ajax_obj.error_ajax_item || 'Error fetching next item'}: ${textStatus} - ${errorThrown}`, 'error');
                rank.helpers.addLogMessage(`AJAX Error during review: ${textStatus} - ${errorThrown}. Response: ${jqXHR.responseText}`);
                
                if (rank.elements.healthIssuesContainer) rank.elements.healthIssuesContainer.show();
                if (rank.elements.processingArea) rank.elements.processingArea.hide();
                if (rank.elements.autoProcessButton) { // Show auto process button again
                    rank.elements.autoProcessButton.show();
                }
                
                // Hide the spinner when there's an AJAX error
                $('#rank-log-spinner').removeClass('is-active').hide();
                
                rank.state.isAutoProcessingActive = false; // Reset mode
            }
        }));
    }
    
    // Initialize the module
    // Initialize the module
    function init() {
        // Clear processing area on page load/tab switch
        if (rank.elements.processingArea) {
            // Hide processing area when plugin is loaded or tab is switched
            rank.elements.processingArea.hide();
            if (rank.elements.healthIssuesContainer) {
                rank.elements.healthIssuesContainer.show();
            }
        }

        // Bind start button click handler
        rank.elements.startButton.on('click', function() {
            // Make sure we have a valid batch ID
            rank.state.batchId = rank.elements.batchIdInput.val() || localStorage.getItem('rank_batch_id');
            if (!rank.state.batchId) {
                rank.helpers.showNotice(rank_ajax_obj.error_no_batch_id, 'error');
                return;
            }
    
            if (rank.state.autoApproveFlag) {
                // This path for startButton should ideally not be hit if autoApproveFlag is true,
                // as user would click "Automatic" button instead.
                // However, if it is, proceed with scheduleAllTasks.
                rank.helpers.addLogMessage('Start button clicked with auto-approve ON - scheduling all tasks.');
                scheduleAllTasks();
            } else {
                // Standard Manual Processing
                rank.state.isAutoProcessingActive = false; // Explicitly set manual mode
                localStorage.setItem('rank_review_started', '1');
    
                rank.elements.startButton.hide(); // Hide self
                if (rank.elements.autoProcessButton) rank.elements.autoProcessButton.hide(); // Hide "Automatic" button
                rank.elements.statusMessage.text(rank_ajax_obj.status_reviewing);
                rank.elements.logOutput.empty();
                rank.state.processedCount = 0;
                if(rank.elements.progressBarContainer) rank.elements.progressBarContainer.show();
                rank.helpers.updateProgressBar(0);
                
                // Get the health issue name based on the current issue type
                let healthIssueName = '';
                if (rank.state.issueType && window.rankLocalizedData && window.rankLocalizedData.healthCheckCategories && window.rankLocalizedData.healthCheckCategories[rank.state.issueType]) {
                    healthIssueName = window.rankLocalizedData.healthCheckCategories[rank.state.issueType] + ': ';
                }
                
                rank.elements.counts.text(`${healthIssueName}${rank_ajax_obj.text_decide_label || 'Processed'}: 1 of ${rank.state.totalCount}`);
                
                // Show the log container when starting manual processing
                $('.rank-log-container').show();
                
                // Hide the spinner for manual processing (we have the loading placeholder now)
                $('#rank-log-spinner').removeClass('is-active').hide();
                
                // Show loading placeholder immediately before fetching the first item
                if (rank.elements.approvalArea) {
                    rank.elements.approvalArea.empty().append(`
                        <div class="rank-approval-row">
                            <div class="rank-loading-placeholder">Loading... 5-15 seconds</div>
                        </div>
                    `).show();
                }
    
                fetchNextRow();
            }
        });
    }
    
    // Expose public methods
    // Expose public methods
    window.rank.processing = {
        init: init,
        scheduleAllTasks: scheduleAllTasks,
        startManualReview: startManualReview,
        fetchNextRow: fetchNextRow,
        resetProcessingState: resetProcessingState,
        startBatchAutoApproval: function() {
            rank.helpers.addLogMessage('Batch auto-approval process initiated.');
            
            // Show and initialize progress bar
            if (rank.elements.progressBarContainer) {
                rank.elements.progressBarContainer.show();
                rank.helpers.updateProgressBar(0);
            }
            
            // Update status message to show auto-processing
            if (rank.elements.statusMessage) {
                rank.elements.statusMessage.text(`Auto-processing: 0 of ${rank.state.totalCount}`);
            }
            
            // Ensure UI is clear for this mode (no single item approval showing from previous manual step)
            if (rank.elements.approvalArea && rank.elements.approvalArea.is(':visible')) {
                 // If an item was shown for manual review before clicking "Automatic", clear it.
                const manualItemVisible = rank.elements.approvalArea.find('.rank-approval-row').length > 0;
                if (manualItemVisible && rank.state.isAutoProcessingActive && rank.state.autoApproveFlag) {
                    rank.elements.approvalArea.empty().hide(); // Hide approval area for automatic processing
                    rank.helpers.addLogMessage('Cleared single item view for batch auto-approval.');
                }
            }
            
            fetchNextRow(); // fetchNextRow will now handle the auto-approval loop if flags are set
        },
        // Clear processing area when switching tabs or reloading
        clearProcessingArea: function() {
            if (rank.elements.processingArea) {
                rank.elements.processingArea.hide();
            }
            if (rank.elements.healthIssuesContainer) {
                rank.elements.healthIssuesContainer.show();
            }
            if (rank.elements.approvalArea) {
                rank.elements.approvalArea.empty().hide(); // Hide approval area when clearing processing area
            }
            // Hide the log container and spinner when clearing processing area
            $('.rank-log-container').hide();
            $('#rank-log-spinner').removeClass('is-active').hide();
        },
        // Cancel all processing and reset state
        cancelProcessing: function() {
            // First, stop any active processing
            rank.state.isAutoProcessingActive = false;
            rank.state.autoProcessActionCompleted = true; // Set to true to prevent further processing
            
            // Reset all state variables and UI
            resetProcessingState();
            
            // If approval module is available, reset its UI
            if (rank.approval && typeof rank.approval.resetApprovalUI === 'function') {
                rank.approval.resetApprovalUI();
            }
            
            // Clear any pending AJAX requests if possible
            if (window.stop) {
                window.stop(); // This attempts to stop pending AJAX requests
            }
            
            // Force a page reload if requested
            if (rank_ajax_obj.force_reload_on_cancel === '1') {
                window.location.reload();
                return;
            }
            
            // Show notification
            rank.helpers.showNotice('Processing canceled. All state has been reset.', 'info');
        }
    };
    // Module initialization complete
    
})(jQuery, window.rank = window.rank || {});

// Assign the module to the rank object
// window.rank.processing = processing; 