<?php
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Get the current domain identifier from the Morningscore API.
 * This is cached in a transient for 24 hours.
 * 
 * @return string|WP_Error Domain identifier or error
 */
function rank_get_domain_identifier() {
    // Check if we have a cached domain identifier
    $domain_identifier = get_transient( 'rank_domain_identifier' );
    if ( $domain_identifier ) {
        return $domain_identifier;
    }

    // Get current domain
     $site_domain = parse_url( home_url(), PHP_URL_HOST );
    
    // Fetch domains from API
    $response = wp_remote_get( 'https://api.morningscore.io/v1/domains', array(
        'headers' => array(
            'Authorization' => 'Bearer ' . rank_get_api_key(),
            'Accept' => 'application/json',
        ),
        'timeout' => 15
    ));

    if ( is_wp_error( $response ) ) {
        return $response;
    }

    $response_code = wp_remote_retrieve_response_code( $response );
    if ( $response_code !== 200 ) {
        return new WP_Error( 
            'api_error', 
            'Failed to retrieve domains. Status code: ' . $response_code
        );
    }

    $domains = json_decode( wp_remote_retrieve_body( $response ), true );
    
    if ( ! is_array( $domains ) ) {
        return new WP_Error( 'api_error', 'Invalid response format when fetching domains' );
    }

    // Find the matching domain
    $domain_identifier = null;
    
    // Normalize the site domain by removing www if present
    $normalized_site_domain = preg_replace( '/^www\./', '', $site_domain );
    
    foreach ( $domains as $domain ) {
        if ( isset( $domain['domain'] ) ) {
            // Normalize the API domain by removing www if present
            $normalized_api_domain = preg_replace( '/^www\./', '', $domain['domain'] );
            
            // Compare the normalized domains
            if ( $normalized_api_domain === $normalized_site_domain ) {
                $domain_identifier = $domain['global_domain_identifier'];
                break;
            }
        }
    }

    if ( ! $domain_identifier ) {
        return new WP_Error( 'domain_not_found', 'Your website domain was not found in your Morningscore account' );
    }

    // Cache the domain identifier for 24 hours
    set_transient( 'rank_domain_identifier', $domain_identifier, DAY_IN_SECONDS );

    return $domain_identifier;
}

/**
 * Get the AI credits status from the Morningscore API.
 * 
 * @return array|WP_Error Array with credit info or error
 */
function rank_get_ai_credits() {
    $response = wp_remote_get( 
        'https://api.morningscore.io/v1/credits/get-status?credit_type=credits.ai.monthly_ai_credits', 
        array(
            'headers' => array(
                'Authorization' => 'Bearer ' . rank_get_api_key(),
                'Accept' => 'application/json',
            ),
            'timeout' => 15
        )
    );

    if ( is_wp_error( $response ) ) {
        return $response;
    }

    $response_code = wp_remote_retrieve_response_code( $response );
    if ( $response_code !== 200 ) {
        return new WP_Error( 
            'api_error', 
            'Failed to retrieve AI credits. Status code: ' . $response_code
        );
    }

    $credits = json_decode( wp_remote_retrieve_body( $response ), true );
    
    if ( ! is_array( $credits ) || ! isset( $credits['limit'] ) || ! isset( $credits['usage'] ) ) {
        return new WP_Error( 'api_error', 'Invalid response format when fetching AI credits' );
    }

    return $credits;
}

/**
 * Get the count of issues for a specific health check category.
 * 
 * @param string $issue_type The category key (e.g., 'missing_alt_tags')
 * @return int|WP_Error The count of issues or error
 */
function rank_get_issue_count( $issue_type ) {
    $domain_identifier = rank_get_domain_identifier();
    if ( is_wp_error( $domain_identifier ) ) {
        return $domain_identifier;
    }

    $url = "https://api.morningscore.io/v1/{$domain_identifier}/get_issues";
    $query_args = array(
        'filter.issue_type' => $issue_type,
        'filter.resolved' => 'false',
        'filter.ignored' => 'false',
        'pagination.page' => 1,
        'pagination.per_page' => 1
    );

    $response = wp_remote_get( 
        add_query_arg( $query_args, $url ),
        array(
            'headers' => array(
                'Authorization' => 'Bearer ' . rank_get_api_key(),
                'Accept' => 'application/json',
            ),
            'timeout' => 15
        )
    );

    if ( is_wp_error( $response ) ) {
        return $response;
    }

    $response_code = wp_remote_retrieve_response_code( $response );
    if ( $response_code !== 200 ) {
        return new WP_Error( 
            'api_error', 
            'Failed to retrieve issue count. Status code: ' . $response_code
        );
    }

    $result = json_decode( wp_remote_retrieve_body( $response ), true );
    
    if ( ! is_array( $result ) || ! isset( $result['total'] ) ) {
        return new WP_Error( 'api_error', 'Invalid response format when fetching issue count' );
    }

    return intval( $result['total'] );
}

/**
 * Get all issues for a specific health check category.
 * Filters out issues with URLs that don't exist in WordPress.
 *
 * @param string $issue_type The category key (e.g., 'missing_alt_tags')
 * @return array|WP_Error Array of issues or error
 */
function rank_get_all_issues( $issue_type ) {
    $domain_identifier = rank_get_domain_identifier();
    if ( is_wp_error( $domain_identifier ) ) {
        return $domain_identifier;
    }

    // First get the total count
    $count = rank_get_issue_count( $issue_type );
    if ( is_wp_error( $count ) ) {
        return $count;
    }

    // Calculate number of pages needed (100 per page)
    $per_page = 100;
    $total_pages = ceil( $count / $per_page );
    $all_issues = array();
    $skipped_count = 0;

    // Fetch all pages
    for ( $page = 1; $page <= $total_pages; $page++ ) {
        $url = "https://api.morningscore.io/v1/{$domain_identifier}/get_issues";
        $query_args = array(
            'filter.issue_type' => $issue_type,
            'filter.resolved' => 'false',
            'filter.ignored' => 'false',
            'pagination.page' => $page,
            'pagination.per_page' => $per_page
        );

        $response = wp_remote_get( 
            add_query_arg( $query_args, $url ),
            array(
                'headers' => array(
                    'Authorization' => 'Bearer ' . rank_get_api_key(),
                    'Accept' => 'application/json',
                ),
                'timeout' => 30
            )
        );

        if ( is_wp_error( $response ) ) {
            return $response;
        }

        $response_code = wp_remote_retrieve_response_code( $response );
        if ( $response_code !== 200 ) {
            return new WP_Error( 
                'api_error', 
                'Failed to retrieve issues. Status code: ' . $response_code
            );
        }

        $result = json_decode( wp_remote_retrieve_body( $response ), true );
        
        if ( ! is_array( $result ) || ! isset( $result['data'] ) || ! is_array( $result['data'] ) ) {
            return new WP_Error( 'api_error', 'Invalid response format when fetching issues' );
        }

        // Filter issues to only include those with URLs that exist in WordPress
        foreach ($result['data'] as $issue) {
            // Get the page URL from the issue data
            $page_url = null;
            if (isset($issue['page']) && isset($issue['page']['url'])) {
                $page_url = $issue['page']['url'];
            } elseif (isset($issue['url'])) {
                $page_url = $issue['url'];
            }

            if (!empty($page_url)) {
                // Check if the URL exists in WordPress
                $wp_post_id = url_to_postid($page_url);
                
                // If standard function fails, try enhanced method for multilingual sites
                if ($wp_post_id <= 0 && class_exists('Rank_Processor') && method_exists('Rank_Processor', 'enhanced_url_to_postid')) {
                    $wp_post_id = Rank_Processor::enhanced_url_to_postid($page_url);
                }
                
                $url_exists = ($wp_post_id > 0);
                
                if ($url_exists) {
                    // URL exists in WordPress, add the issue to the list
                    $all_issues[] = $issue;
                } else {
                    // URL doesn't exist in WordPress, skip this issue
                    $skipped_count++;
                }
            } else {
                // No URL found in the issue data, add it anyway
                $all_issues[] = $issue;
            }
        }

        // Add a short delay between requests
        if ( $page < $total_pages - 1 ) {
            sleep( 0.2 );
        }
    }


    return $all_issues;
}

/**
 * Get AI fix for a specific issue.
 * 
 * @param string $rank_id The rank ID of the issue
 * @param string $domain_identifier Optional domain identifier (if not provided, it will be fetched)
 * @return array|WP_Error The AI fix or error
 */
function rank_get_ai_fix( $rank_id, $domain_identifier = null ) {
    if ( ! $domain_identifier ) {
        $domain_identifier = rank_get_domain_identifier();
        if ( is_wp_error( $domain_identifier ) ) {
            return $domain_identifier;
        }
    }

    $url = "https://api.morningscore.io/v1/{$domain_identifier}/get_ai_fix";
    
    $request_body = array(
        'rank_id' => $rank_id,
        'fresh_data' => null
    );
    
    $response = wp_remote_post(
        $url,
        array(
            'headers' => array(
                'Authorization' => 'Bearer ' . rank_get_api_key(),
                'Content-Type' => 'application/json',
                'Accept' => 'application/json',
            ),
            'body' => wp_json_encode($request_body),
            'timeout' => 60 // Longer timeout for AI processing
        )
    );

    if ( is_wp_error( $response ) ) {
        return $response;
    }

    $response_code = wp_remote_retrieve_response_code( $response );
    $response_body = wp_remote_retrieve_body( $response );
    
    $decoded_body = json_decode( $response_body, true );

    if ( $response_code !== 200 ) {
        // Specifically check for "OUT_OF_CREDITS"
        if ( isset( $decoded_body['message'] ) && $decoded_body['message'] === 'OUT_OF_CREDITS' ) {
            return new WP_Error(
                'out_of_credits',
                'AI credits exhausted. API Message: OUT_OF_CREDITS',
                array( 'status_code' => $response_code ) // Pass status code
            );
        }
        
        // Handle other API errors
        $error_message = isset( $decoded_body['message'] ) ? $decoded_body['message'] : 'Failed to retrieve AI fix.';
        return new WP_Error(
            'api_error',
            "Failed to retrieve AI fix. Status code: $response_code. Message: $error_message",
            array( 'status_code' => $response_code ) // Pass status code
        );
    }

    // Check if the response has the expected content field
    if ( ! is_array( $decoded_body ) ) {
        return new WP_Error(
            'api_error',
            'Invalid response format when fetching AI fix - not an array.',
            array( 'status_code' => $response_code )
        );
    }
    
    // Handle different response formats
    if ( ! isset( $decoded_body['content'] ) ) {
        return new WP_Error(
            'api_error',
            'The AI had an error. Skipping this fix.',
            array( 'status_code' => $response_code, 'response' => $decoded_body )
        );
    }
    
    return $decoded_body; // Success, $decoded_body should contain 'content'
}

/**
 * Get health issue details by category.
 * This function caches the issue counts for all categories.
 * 
 * @return array|WP_Error Array with issue counts by category or error
 */
function rank_get_health_issue_counts() {
    $all_categories_config = rank_get_health_check_categories(); // Get all defined categories
    $issue_counts_to_return = array();

    foreach ( $all_categories_config as $key => $label ) {
        $actual_count_value = rank_get_issue_count( $key ); // This function should return an integer or WP_Error.

        $current_issue_count = 0; // Default to 0

        if ( is_wp_error( $actual_count_value ) ) {
            // WP_Error getting issue count
            // Keep count as 0
        } elseif ( ! is_numeric( $actual_count_value ) ) {
            // Non-numeric issue count received
            // Keep count as 0
        } else {
            $current_issue_count = (int) $actual_count_value;
        }
        
        $issue_counts_to_return[$key] = array(
            'label' => $label, // Use the label from the central config
            'count' => $current_issue_count < 0 ? 0 : $current_issue_count // Ensure count is not negative
        );
    }

    return $issue_counts_to_return;
}

/**
 * Check if an issue has already been fixed locally.
 *
 * @param string $issue_type The type of issue (e.g., 'missing_alt_tags')
 * @param string $page_url The URL of the page with the issue
 * @param string $issue_identifier The specific identifier for the issue (e.g., image URL)
 * @return bool True if the issue has already been fixed locally
 */
function rank_is_issue_already_fixed($issue_type, $page_url, $issue_identifier) {
    // Static caches to avoid repeated lookups and database queries
    static $attachment_id_cache = [];
    static $fixed_status_cache = [];
    static $post_id_cache = [];
    static $option_cache = [];
    
    // Debug log the function call
    // Check if issue is fixed
    
    // Create a unique cache key for this check
    $cache_key = md5($issue_type . '|' . $page_url . '|' . $issue_identifier);
    
    // Check if we already have a cached result
    if (isset($fixed_status_cache[$cache_key])) {
        // Using cached result
        return $fixed_status_cache[$cache_key];
    }
    
    // Different logic based on issue type
    $issue_config = rank_get_issue_type_config($issue_type);
    
    if (!$issue_config) {
        // No issue config found
        $fixed_status_cache[$cache_key] = false;
        return false;
    }
    
    // Process issue config
    
    // Get post ID from URL (with caching)
    if (isset($post_id_cache[$page_url])) {
        $post_id = $post_id_cache[$page_url];
    } else {
        $post_id = url_to_postid($page_url);
        if (!$post_id) {
            // Try enhanced URL to post ID conversion
            $post_id = Rank_Processor::enhanced_url_to_postid($page_url);
        }
        $post_id_cache[$page_url] = $post_id;
    }
    
    if ($issue_config['handler_type'] === 'database_override') {
        global $wpdb;
        $table_name = $wpdb->prefix . RANK_TABLE_NAME;
        $db_override_type = '';
        
        // Map issue type to database override type
        switch ($issue_type) {
            case 'missing_meta_description':
            case 'short_meta_description':
            case 'long_meta_description':
                $db_override_type = 'meta_description';
                break;
            case 'missing_title_tag':
            case 'short_title_tag':
            case 'long_title_tag':
                $db_override_type = 'title';
                break;
            case 'missing_alt_tags':
                $db_override_type = 'image_alt';
                
                // For alt tags, use a more efficient approach
                if ($post_id > 0) {
                    // First check if this post has any alt tag overrides at all
                    // This is much faster than looking up each attachment ID
                    static $post_has_alt_overrides = [];
                    
                    if (!isset($post_has_alt_overrides[$post_id])) {
                        $query = $wpdb->prepare(
                            "SELECT COUNT(*) FROM `$table_name` WHERE post_id = %d AND override_type = %s LIMIT 1",
                            $post_id, $db_override_type
                        );
                        $count = $wpdb->get_var($query);
                        $post_has_alt_overrides[$post_id] = ($count > 0);
                    }
                    
                    // If the post has any alt tag overrides, we'll consider the issue fixed
                    // This is a simplification that avoids the expensive attachment ID lookup
                    if ($post_has_alt_overrides[$post_id]) {
                        $fixed_status_cache[$cache_key] = true;
                        return true;
                    }
                    
                    // Only if we need exact matching, do the more expensive lookup
                    if ($issue_config['identifier_is_url'] && !empty($issue_identifier) && defined('RANK_EXACT_ALT_MATCHING') && RANK_EXACT_ALT_MATCHING) {
                        // Use cached attachment ID if available
                        if (!isset($attachment_id_cache[$issue_identifier])) {
                            $attachment_id_cache[$issue_identifier] = Rank_Processor::get_attachment_id_from_url($issue_identifier);
                        }
                        $object_id = $attachment_id_cache[$issue_identifier];
                        
                        if ($object_id > 0) {
                            // Check if this specific image alt tag has been fixed
                            $query = $wpdb->prepare(
                                "SELECT COUNT(*) FROM `$table_name` WHERE post_id = %d AND object_id = %d AND override_type = %s",
                                $post_id, $object_id, $db_override_type
                            );
                            $count = $wpdb->get_var($query);
                            $fixed_status_cache[$cache_key] = ($count > 0);
                            return $fixed_status_cache[$cache_key];
                        }
                    }
                }
                
                $fixed_status_cache[$cache_key] = false;
                return false;
        }
        
        // For non-image issues or if image ID wasn't found
        if ($post_id > 0 && !empty($db_override_type)) {
            $query = $wpdb->prepare(
                "SELECT COUNT(*) FROM `$table_name` WHERE post_id = %d AND override_type = %s",
                $post_id, $db_override_type
            );
            $count = $wpdb->get_var($query);
            // Database query executed
            $fixed_status_cache[$cache_key] = ($count > 0);
            return $fixed_status_cache[$cache_key];
        } else {
            // Skipping database query
        }
    }
    elseif ($issue_config['handler_type'] === 'option_suggestion') {
        // For broken links
        $option_name = '';
        if ($issue_type === 'broken_links_internal') {
            $option_name = 'rank_internal_url_replacements';
        } elseif ($issue_type === 'broken_links_external') {
            $option_name = 'rank_external_url_replacements';
        }
        
        // Checking option suggestion
        
        if (!empty($option_name) && !empty($issue_identifier)) {
            // Cache option values to avoid repeated get_option calls
            if (!isset($option_cache[$option_name])) {
                $option_cache[$option_name] = get_option($option_name, []);
                // Option loaded
            }
            $replacements = $option_cache[$option_name];
            
            // Check if this specific URL has a replacement
            $result = (is_array($replacements) && isset($replacements[$issue_identifier]));
            // URL replacement check completed
            $fixed_status_cache[$cache_key] = $result;
            return $result;
        } else {
            // Skipping option check
        }
    }
    
    $fixed_status_cache[$cache_key] = false;
    return false;
}

/**
 * Get health issue details by category with information about already fixed issues.
 * Uses sampling for performance and makes only one API call per issue type.
 *
 * @return array|WP_Error Array with issue counts by category or error
 */
function rank_get_health_issue_counts_with_fix_status() {
    // Tell PHP to stop execution when the client disconnects
    ignore_user_abort(false);
    
    // Set a reasonable timeout for this function
    set_time_limit(30);
    
    $all_categories = rank_get_health_check_categories();
    $issue_counts = array();
    $default_sample_size = 5; // Default number of issues to sample for each type
    
    // Get domain identifier once for all categories
    $domain_identifier = rank_get_domain_identifier();
    
    if (is_wp_error($domain_identifier)) {
        return $domain_identifier;
    }
    
    // Set a reasonable timeout to ensure complete data fetching
    // Increased from 3 seconds to 10 seconds to prevent incomplete data
    $api_timeout = 10;
    
    // Array to store timing data for each issue type
    $timing_data = array();
    
    foreach ($all_categories as $issue_type => $label) {
        // Use different sample sizes for specific issue types
        $sample_size = $default_sample_size;
        if ($issue_type === 'missing_alt_tags') {
            $sample_size = 3;
        }
        
        // Make a single API call to get both count and sample data
        $url = "https://api.morningscore.io/v1/{$domain_identifier}/get_issues";
        $query_args = array(
            'filter.issue_type' => $issue_type,
            'filter.resolved' => 'false',
            'filter.ignored' => 'false',
            'pagination.page' => 1,
            'pagination.per_page' => $sample_size // Get a sample of issues
        );

        // Make API call

        $response = wp_remote_get(
            add_query_arg($query_args, $url),
            array(
                'headers' => array(
                    'Authorization' => 'Bearer ' . rank_get_api_key(),
                    'Accept' => 'application/json',
                ),
                'timeout' => $api_timeout,
                'blocking' => true
            )
        );
        
        // API call completed
        
        if (is_wp_error($response)) {
            $issue_counts[$issue_type] = array(
                'label' => $label,
                'count' => 0,
                'has_fixed_issues' => false,
                'sample_size' => 0,
                'sample_fixed_count' => 0,
                'needs_recrawl' => false
            );
            continue;
        }

        $response_code = wp_remote_retrieve_response_code($response);
        if ($response_code !== 200) {
            $issue_counts[$issue_type] = array(
                'label' => $label,
                'count' => 0,
                'has_fixed_issues' => false,
                'sample_size' => 0,
                'sample_fixed_count' => 0,
                'needs_recrawl' => false
            );
            continue;
        }
        
        $result = json_decode(wp_remote_retrieve_body($response), true);
        
        // Process API response
        
        if (!is_array($result) || !isset($result['total']) || !isset($result['data']) || !is_array($result['data'])) {
            $issue_counts[$issue_type] = array(
                'label' => $label,
                'count' => 0,
                'has_fixed_issues' => false,
                'sample_size' => 0,
                'sample_fixed_count' => 0,
                'needs_recrawl' => false
            );
            continue;
        }

        // Extract the total count from the response
        $count = intval($result['total']);
        
        // Check if any issues in the sample have been fixed locally
        $fixed_count = 0;
        $sample_issues = $result['data'];
        
        if ($count > 0 && !empty($sample_issues)) {
            // Pre-process all issues to collect data for batch processing
            $issues_to_check = [];
            // Process sample issues
            
            foreach ($sample_issues as $issue) {
                // Extract page URL - try different possible structures
                $page_url = '';
                if (isset($issue['page']['path_full'])) {
                    $page_url = $issue['page']['path_full'];
                } elseif (isset($issue['page']['url'])) {
                    $page_url = $issue['page']['url'];
                } elseif (isset($issue['url'])) {
                    $page_url = $issue['url'];
                } elseif (isset($issue['page_url'])) {
                    $page_url = $issue['page_url'];
                } elseif (isset($issue['page']) && is_string($issue['page'])) {
                    $page_url = $issue['page'];
                }
                
                // Extract issue identifier - try different possible structures
                $issue_identifier = '';
                if (isset($issue['issue_identifier'])) {
                    $issue_identifier = $issue['issue_identifier'];
                } elseif (isset($issue['identifier'])) {
                    $issue_identifier = $issue['identifier'];
                } elseif (isset($issue['id'])) {
                    $issue_identifier = $issue['id'];
                }
                
                // Store extracted data
                
                // Store the issue data for batch processing
                $issues_to_check[] = [
                    'page_url' => $page_url,
                    'issue_identifier' => $issue_identifier
                ];
            }
            
            // Process all issues in a batch
            foreach ($issues_to_check as $issue_data) {
                // Check if this issue has been fixed locally
                if (rank_is_issue_already_fixed($issue_type, $issue_data['page_url'], $issue_data['issue_identifier'])) {
                    $fixed_count++;
                }
            }
        }
        
        // Store the results
        $issue_counts[$issue_type] = array(
            'label' => $label,
            'count' => $count < 0 ? 0 : $count, // Ensure count is not negative
            'has_fixed_issues' => ($fixed_count > 0),
            'sample_size' => count($sample_issues),
            'sample_fixed_count' => $fixed_count,
            'needs_recrawl' => ($fixed_count > 0)
        );
        
    }
    
    // Return the results
    
    return $issue_counts;
}

/**
 * Clear the health issues cache.
 * This should be called whenever an issue is fixed to ensure fresh data.
 */
function rank_clear_health_issues_cache() {
    delete_transient('rank_health_issue_counts_with_fix_status');
    delete_transient('rank_health_issue_counts');
}
