<?php
/**
 * Handles AJAX requests for the RANK AI plugin.
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

class Rank_Ajax {

    /**
     * Initialize AJAX hooks.
     */
    public static function init() {
        // Action hooks for the API-based workflow
        add_action( 'wp_ajax_rank_start_processing', array( __CLASS__, 'ajax_start_processing' ) );
        add_action( 'wp_ajax_rank_delete_all_data', array( __CLASS__, 'ajax_delete_all_data' ) );
        add_action( 'wp_ajax_rank_get_fixes_data', array( __CLASS__, 'ajax_get_fixes_data' ) );
        add_action( 'wp_ajax_rank_get_all_fix_counts', array( __CLASS__, 'ajax_get_all_fix_counts' ) );
        add_action( 'wp_ajax_rank_prepare_health_issues', array( __CLASS__, 'ajax_prepare_health_issues' ) );
        add_action( 'wp_ajax_rank_get_batch_state', array( __CLASS__, 'ajax_get_batch_state' ) );
        add_action( 'wp_ajax_rank_record_decision', array( __CLASS__, 'ajax_record_decision' ) );
        add_action( 'wp_ajax_rank_schedule_tasks', array( __CLASS__, 'ajax_schedule_tasks' ) );
        add_action( 'wp_ajax_rank_get_health_issues_overview', array( __CLASS__, 'ajax_get_health_issues_overview' ) );
        add_action( 'wp_ajax_rank_clear_pending_decisions', array( __CLASS__, 'ajax_clear_pending_decisions' ) );
        add_action( 'wp_ajax_rank_update_fix_value', array( __CLASS__, 'ajax_update_fix_value' ) );
        
        // Set a global timeout for all AJAX requests
        add_action( 'wp_ajax_nopriv_', array( __CLASS__, 'setup_abort_handling' ) );
        add_action( 'wp_ajax_', array( __CLASS__, 'setup_abort_handling' ) );
    }
    
    /**
     * Set up abort handling for all AJAX requests
     */
    public static function setup_abort_handling() {
        // Tell PHP to stop execution when the client disconnects
        ignore_user_abort(false);
        
        // Set a reasonable timeout for all AJAX requests
        set_time_limit(30);
        
        // Set up AJAX request handling
    }

    /**
     * AJAX handler for scheduling tasks from batch data.
     */
    public static function ajax_schedule_tasks() {
        check_ajax_referer( 'rank_ajax_nonce', 'nonce' );

        if ( ! isset( $_POST['batch_id'] ) || empty( $_POST['batch_id'] ) ) {
            wp_send_json_error( array( 'message' => 'Missing batch ID.' ) );
        }

        $batch_id = sanitize_key( $_POST['batch_id'] );
        if ( strpos($batch_id, 'rank_batch_') !== 0 ) {
             wp_send_json_error( array( 'message' => 'Invalid batch ID format.' ) );
        }

        $batch_data = get_option( $batch_id );

        if ( false === $batch_data || ! is_array( $batch_data ) || !isset($batch_data['issues']) || empty( $batch_data['issues'] ) || !isset($batch_data['issue_type']) ) {
             wp_send_json_error( array( 'message' => 'Batch data not found, invalid, or empty. Please prepare the batch again.' ) );
        }

        $issues = $batch_data['issues'];
        $issue_type = $batch_data['issue_type']; // Already checked isset above
        $action_group = 'rank-tasks-' . $batch_id;

        delete_transient( 'rank_log_' . $batch_id );

        $scheduled_count = 0;
        $skipped_count = 0; // Add counter for skipped rows
        
        foreach ( $issues as $index => $issue ) { 
            // Skip issues that don't have decisions or were declined
            $row_key = $issue['rank_id']; // rank_id is the unique key for an issue in the batch
            if (!isset($batch_data['decisions'][$row_key]) || 
                !$batch_data['decisions'][$row_key]['approved']) {
                $skipped_count++;
                continue;
            }
            
            $decision = $batch_data['decisions'][$row_key];
            
            // Log the issue being processed

            // Validate the essential data exists
            // Page ID is crucial for context.
            if (!isset($issue['page_id']) || empty($issue['page_id'])) {
                // Missing page_id in batch
                $skipped_count++;
                continue;
            }
            
            // The specific URL of the item to be fixed (e.g., image URL)
            // Prefer 'issue_identifier' as it's directly tied to the issue, fallback to 'parameters.url'
            $original_item_url = $issue['issue_identifier'] ?? ($issue['parameters']['url'] ?? null);
            if (empty($original_item_url)) {
                // Missing issue_identifier or parameters.url
                $skipped_count++;
                continue;
            }
            $original_item_url = esc_url_raw(trim($original_item_url));


            $solution = isset($decision['solution']) ? trim($decision['solution']) : null;
            
            // Skip if solution is empty
            if (empty($solution)) {
                // Missing solution
                $skipped_count++;
                continue;
            }

            // If we reach here, validation passed for this issue
            try {
                $task_args = array(
                    'page_id'      => $issue['page_id'], // Contextual page ID
                    'object_id'    => $issue['page_id'], // Initial object_id, Rank_Processor will refine for images
                    'issue_type'   => $issue_type,       // The overall issue type for the batch
                    'solution'     => $solution,
                    'original_url' => $original_item_url, // URL of the specific item (e.g. image)
                    'row_key'      => $row_key,
                    'batch_id'     => $batch_id,
                );


                $action_id = as_enqueue_async_action(
                    'rank_process_single_item_hook', // Use the same hook as ajax_record_decision
                    array( $task_args ),             // Pass arguments as a single array
                    $action_group                    // Action group
                );

                if ( $action_id ) {
                    $scheduled_count++;
                } else {
                    $skipped_count++; // Consider it skipped if scheduling failed
                    // Failed to schedule task
                }
            } catch ( Exception $e ) {
                $skipped_count++;
                // Error scheduling task
            }
        }

        // Response message
        if ( $scheduled_count > 0 ) {
            $message = $scheduled_count . ' tasks scheduled successfully.';
            if ($skipped_count > 0) {
                $message .= ' ' . $skipped_count . ' items were skipped (check PHP error logs for details).';
            }
            wp_send_json_success( array( 'message' => $message ) );
        } else {
             $message = 'No tasks were scheduled.';
             if ($skipped_count > 0) {
                 $message .= ' All ' . $skipped_count . ' items were skipped (check PHP error logs for details).';
             } else {
                 $message .= ' The batch might be empty or contain no approved fixes.';
             }
             wp_send_json_error( array( 'message' => $message ) );
        }
    }

    /**
     * AJAX handler for deleting all override data (custom table and URL replacement option).
     */
    public static function ajax_delete_all_data() {
        check_ajax_referer( 'rank_ajax_nonce', 'nonce' );
        if ( ! current_user_can( 'manage_options' ) ) {
            wp_send_json_error( array( 'message' => 'Permission denied.' ), 403 );
        }

        global $wpdb;
        $table_name = $wpdb->prefix . RANK_TABLE_NAME;
        $table_truncated = false;
        $options_deleted = true; // Assume success unless one fails
        $db_error = ''; // Initialize error variable

        // Attempt to truncate the custom table
        // Ensure RANK_TABLE_NAME is defined
        if ( defined('RANK_TABLE_NAME') && !empty(RANK_TABLE_NAME) ) { // Basic check
            $result = $wpdb->query( "TRUNCATE TABLE `$table_name`" );
            if ( $result !== false ) {
                $table_truncated = true;
            } else {
                $db_error = $wpdb->last_error; // Capture error before next operation
            }
        } else {
             $db_error = 'RANK_TABLE_NAME constant not defined or invalid.';
             // Error in delete_all_overrides
        }

        // Initialize options_deleted as true (assume success)
        $options_deleted = true;
        
        // For internal URL replacements
        $internal_option = get_option('rank_internal_url_replacements', false);
        if ($internal_option !== false) {
            // Option exists, try to delete it
            if (!delete_option('rank_internal_url_replacements')) {
                $options_deleted = false;
                // Failed to delete rank_internal_url_replacements option
            }
        }
        
        // For external URL replacements
        $external_option = get_option('rank_external_url_replacements', false);
        if ($external_option !== false) {
            // Option exists, try to delete it
            if (!delete_option('rank_external_url_replacements')) {
                $options_deleted = false;
                // Failed to delete rank_external_url_replacements option
            }
        }

        if ( $table_truncated && $options_deleted ) {
            wp_send_json_success( array( 'message' => 'All RANK AI data deleted successfully.' ) );
        } else {
            $error_message = 'Error deleting data.';
            if (!$table_truncated) {
                $error_message .= ' Failed to truncate table.';
                if ($db_error) {
                     $error_message .= ' DB Error: ' . $db_error;
                }
            }
            if (!$options_deleted) {
                 $error_message .= ' Failed to delete one or more URL replacement options.';
            }
            wp_send_json_error( array( 'message' => $error_message ), 500 );
        }
    }

    /**
     * AJAX handler to initiate the processing for a given batch ID.
     * This confirms the batch is ready and allows the frontend to start polling for status.
     */
    public static function ajax_start_processing() {
        check_ajax_referer( 'rank_ajax_nonce', 'nonce' );

        if ( ! current_user_can( 'manage_options' ) ) {
            wp_send_json_error( 'Permission denied.', 403 );
        }

        if ( ! isset( $_POST['batch_id'] ) || empty( $_POST['batch_id'] ) ) {
            wp_send_json_error( 'Missing batch ID.', 400 );
        }

        $batch_id = sanitize_key( $_POST['batch_id'] );

        if ( strpos($batch_id, 'rank_batch_') !== 0 ) {
             wp_send_json_error( 'Invalid batch ID format.', 400 );
        }
        
        // Verify batch exists
        $batch_data = get_option( $batch_id );
        if (!$batch_data) {
            wp_send_json_error( 'Batch not found.', 404 );
        }

        // If checks pass, send success to allow JS polling to begin
        wp_send_json_success( 'Processing initiated. Status checks will begin.' );
    }

    /**
     * AJAX handler to fetch the total count of applied fixes for ALL categories.
     */
    public static function ajax_get_all_fix_counts() {
        check_ajax_referer( 'rank_ajax_nonce', 'nonce' );

        if ( ! current_user_can( 'manage_options' ) ) {
            wp_send_json_error( 'Permission denied.', 403 );
        }

        $categories = rank_get_health_check_categories();
        $counts = [];
        global $wpdb;
        $table_name = $wpdb->prefix . RANK_TABLE_NAME;

        // --- Map frontend key to DB type ---
        $db_category_map = [
            'missing_meta_description' => 'meta_description',
            'short_meta_description'   => 'meta_description',
            'long_meta_description'    => 'meta_description',
            'missing_title_tag'        => 'title',
            'short_title_tag'          => 'title',
            'long_title_tag'           => 'title',
            'missing_alt_tags'         => 'image_alt',
        ];
        // --- End Mapping ---

        // Define which categories use the DB vs Options
        $db_override_keys = array_keys($db_category_map);
        $option_override_keys = [ 'broken_internal_links', 'broken_external_links' ];
        
        // Map frontend keys to option names for broken links
        $option_name_map = [
            'broken_internal_links' => 'rank_internal_url_replacements',
            'broken_external_links' => 'rank_external_url_replacements',
            'broken_links_internal' => 'rank_internal_url_replacements',
            'broken_links_external' => 'rank_external_url_replacements'
        ];

        foreach ( array_keys( $categories ) as $key ) {
            if ( in_array( $key, $db_override_keys ) ) {
                // Get the corresponding database type from the map
                $db_override_type = isset($db_category_map[$key]) ? $db_category_map[$key] : null;

                if ($db_override_type) {
                    // Get count from Custom Table using the mapped type AND issue_type
                    $count = $wpdb->get_var( $wpdb->prepare(
                        "SELECT COUNT(*) FROM `$table_name` WHERE override_type = %s AND (issue_type = %s OR (issue_type IS NULL AND %s = %s))",
                        $db_override_type,
                        $key,
                        $key,
                        $db_override_type
                    ) );
                    $counts[ $key ] = ( $count !== null ) ? absint( $count ) : 0;
                } else {
                     $counts[ $key ] = 0;
                     error_log("Failed to map DB category key: " . $key);
                }

            } elseif ( in_array( $key, $option_override_keys ) || isset($option_name_map[$key]) ) {
                // Get count from Options Table
                $option_name = isset($option_name_map[$key]) ? $option_name_map[$key] : null;
                
                if ($option_name) {
                    $rules = get_option( $option_name, [] );
                    $count = is_array( $rules ) ? count( $rules ) : 0;
                    
                    // Store the count under both the frontend key and the option key
                    $counts[ $key ] = $count;
                    
                    // If this is a broken_links_* key, also store under broken_*_links for backwards compatibility
                    if ($key === 'broken_links_internal') {
                        $counts['broken_internal_links'] = $count;
                    } elseif ($key === 'broken_links_external') {
                        $counts['broken_external_links'] = $count;
                    }
                    
                    // If this is a broken_*_links key, also store under broken_links_* for forwards compatibility
                    if ($key === 'broken_internal_links') {
                        $counts['broken_links_internal'] = $count;
                    } elseif ($key === 'broken_external_links') {
                        $counts['broken_links_external'] = $count;
                    }
                } else {
                    $counts[ $key ] = 0;
                    error_log("Failed to map option name for key: " . $key);
                }
            } else {
                // Handle potential future categories or unknown types
                $counts[ $key ] = 0;
            }
        }

        wp_send_json_success( $counts );
    }

    /**
     * AJAX handler to get data for the "Fixes Overview" tab.
     */
    public static function ajax_get_fixes_data() {
        check_ajax_referer( 'rank_ajax_nonce', 'nonce' );

        if ( ! current_user_can( 'manage_options' ) ) {
            wp_send_json_error( 'Permission denied.', 403 );
        }

        $category_key = isset( $_POST['category'] ) ? sanitize_key( $_POST['category'] ) : null;
        $issue_type   = isset( $_POST['issue_type'] ) ? sanitize_key( $_POST['issue_type'] ) : $category_key;
        $page         = isset( $_POST['page'] ) ? absint( $_POST['page'] ) : 1;
        $per_page     = 20; // Or make this configurable
        $offset       = ( $page - 1 ) * $per_page;

        if ( ! $category_key ) {
            wp_send_json_error( 'Category not specified.', 400 );
        }

        $results      = [];
        $total_items  = 0;
        
        // Dynamically determine if the category uses database overrides or option suggestions
        $all_issue_configs = rank_get_issue_type_config();
        $current_category_config = $all_issue_configs[$category_key] ?? null;

        if (!$current_category_config) {
            error_log("Unknown category key: " . $category_key);
            wp_send_json_error( 'Invalid category specified.', 400 );
            return;
        }

        $handler_type = $current_category_config['handler_type'] ?? null;
        $db_override_type = null; // For database_override types

        if ($handler_type === 'database_override') {
            // Map API issue type to database override_type
            // This mapping logic should align with Rank_Processor
            switch ( $category_key ) {
                case 'missing_meta_description':
                case 'short_meta_description':
                case 'long_meta_description':
                    $db_override_type = 'meta_description';
                    break;
                case 'missing_title_tag':
                case 'short_title_tag':
                case 'long_title_tag':
                    $db_override_type = 'title';
                    break;
                case 'missing_alt_tags':
                case 'image_alt': // Add this case to handle direct image_alt category key
                    $db_override_type = 'image_alt';
                    break;
                // Add other database_override types if any
            }

            if ($db_override_type) {
                global $wpdb;
                $table_name = $wpdb->prefix . 'rank_seo_overrides';

                // Use the provided issue_type for filtering
                $issue_type_filter = $issue_type;
                
                // For 'image_alt' category key, use 'missing_alt_tags' as the issue_type filter if not specified
                if ($category_key === 'image_alt' && $issue_type === $category_key) {
                    $issue_type_filter = 'missing_alt_tags';
                }
                
                $total_items_query = $wpdb->prepare(
                    "SELECT COUNT(DISTINCT override_id) FROM {$table_name} WHERE override_type = %s AND (issue_type = %s OR (issue_type IS NULL AND %s = %s))",
                    $db_override_type,
                    $issue_type_filter,
                    $issue_type_filter,
                    $db_override_type
                );
                $total_items = (int) $wpdb->get_var( $total_items_query );

                $query = $wpdb->prepare(
                    "SELECT post_id, object_id, override_value, issue_type FROM {$table_name} WHERE override_type = %s AND (issue_type = %s OR (issue_type IS NULL AND %s = %s)) ORDER BY override_id DESC LIMIT %d OFFSET %d",
                    $db_override_type,
                    $issue_type_filter,
                    $issue_type_filter,
                    $db_override_type,
                    $per_page,
                    $offset
                );
                $db_results = $wpdb->get_results( $query );


                // Format results
                if ( $db_results ) {
                    foreach ( $db_results as $row ) {
                        $context_url = $row->post_id ? get_permalink( $row->post_id ) : null;
                        $item_details = [
                            'context' => $context_url ? esc_url($context_url) : 'Post ID: ' . $row->post_id,
                            'value'   => esc_html( $row->override_value ),
                            'post_id' => $row->post_id,
                            'object_id' => $row->object_id,
                            'issue_type' => isset($row->issue_type) ? $row->issue_type : $category_key,
                        ];
                        if ($db_override_type === 'image_alt' && $row->object_id > 0) {
                             $attachment_url = wp_get_attachment_url($row->object_id);
                             $item_details['image_src'] = $attachment_url ? esc_url($attachment_url) : 'Attachment ID: ' . $row->object_id;
                             $item_details['context_text'] = ($context_url ? get_the_title($row->post_id) : ($row->post_id ? 'Post ID: ' . $row->post_id : 'Site-wide')) . ' (Image ID: ' . $row->object_id . ')';
                        } else {
                            $item_details['context_text'] = $context_url ? get_the_title($row->post_id) : ($row->post_id ? 'Post ID: ' . $row->post_id : 'N/A');
                        }
                         $results[] = $item_details;
                    }
                }
            } else {
                 // Category has handler_type 'database_override' but no matching db_override_type
            }

        } elseif ( $handler_type === 'option_suggestion' ) {
            // Get rules from Options Table
            $option_name = null;
            if ($category_key === 'broken_links_internal') { // Use new key
                $option_name = 'rank_internal_url_replacements';
            } elseif ($category_key === 'broken_links_external') { // Use new key
                $option_name = 'rank_external_url_replacements';
            }
            // Add other option_suggestion types if any

            if ($option_name) {
                $rules = get_option( $option_name, [] );
                $total_items = is_array( $rules ) ? count( $rules ) : 0;

                if ( is_array( $rules ) && $total_items > 0 ) {
                    // Manual pagination for option arrays
                    $paginated_rules = array_slice( $rules, $offset, $per_page, true );

                    foreach ( $paginated_rules as $old_url => $new_url_data ) {
                        // Assuming $new_url_data might be an array with 'new_url' and 'context'
                        $suggestion_value = '';
                        $landing_page_url = ''; // Will store the landing page URL
                        
                        if (is_array($new_url_data)) {
                            // Get the new URL (replacement)
                            if (isset($new_url_data['new_url'])) {
                                $suggestion_value = $new_url_data['new_url'];
                            }
                            
                            // Get the landing page URL (where the broken link was found)
                            if (isset($new_url_data['context'])) {
                                $landing_page_url = $new_url_data['context'];
                            } elseif (isset($new_url_data['context_post_id'])) {
                                // Legacy format with context_post_id
                                $context_post_id = absint($new_url_data['context_post_id']);
                                $page_title = get_the_title($context_post_id);
                                $page_link = get_permalink($context_post_id);
                                if ($page_title && $page_link) {
                                    $landing_page_url = $page_link;
                                }
                            }
                        } elseif (is_string($new_url_data)) {
                            $suggestion_value = $new_url_data; // Legacy format
                        }

                        $results[] = [
                            'context'      => $landing_page_url ?: $old_url, // Landing page URL (where the broken link was found)
                            'value'        => esc_html( $suggestion_value ), // New URL (replacement)
                            'context_text' => $old_url, // Old URL (the broken link)
                            'is_option'    => true,
                        ];
                    }
                }
            } else {
                // Category has handler_type 'option_suggestion' but no matching option_name
            }
        } else {
            // Unknown handler_type for category key
        }

        // Prepare response data
        $response_data = [
            'items'        => $results,
            'total_items'  => $total_items,
            'per_page'     => $per_page,
            'current_page' => $page,
            'total_pages'  => $total_items > 0 ? ceil( $total_items / $per_page ) : 0,
            'category'     => $category_key,
            'issue_type'   => $issue_type,
        ];

        wp_send_json_success( $response_data );
    }

    /**
     * AJAX handler for preparing health issues for a specific category.
     * This fetches all items for the issue type and creates a batch.
     */
    public static function ajax_prepare_health_issues() {
        // Tell PHP to stop execution when the client disconnects
        ignore_user_abort(false);
        
        // Set a reasonable timeout for this specific request
        set_time_limit(15);
        
        check_ajax_referer( 'rank_ajax_nonce', 'nonce' );

        if ( ! current_user_can( 'manage_options' ) ) {
            wp_send_json_error( 'Permission denied.', 403 );
        }

        $issue_type = isset( $_POST['issue_type'] ) ? sanitize_key( $_POST['issue_type'] ) : '';
        if ( empty( $issue_type ) ) {
            wp_send_json_error( 'Issue type not specified.', 400 );
        }

        // Validate issue type
        $categories = rank_get_health_check_categories();
        if ( ! array_key_exists( $issue_type, $categories ) ) {
            wp_send_json_error( 'Invalid issue type.', 400 );
        }

        // Get all issues for this category (this returns a numerically indexed list)
        $raw_issues_list = rank_get_all_issues( $issue_type );
        if ( is_wp_error( $raw_issues_list ) ) {
            wp_send_json_error( $raw_issues_list->get_error_message(), 400 );
        }

        // No issues to process
        if ( empty( $raw_issues_list ) ) {
            // Send success with count 0, so UI can handle "no issues" gracefully
            wp_send_json_success( array(
                'batchId' => null, // No batch created if no issues
                'count' => 0,
                'message' => 'No issues found for this category.'
            ) );
            return;
        }

        // Prepare an associative array map of issues, keyed by their rank_id
        $prepared_issues_map = [];
        foreach ($raw_issues_list as $issue_item) {
            // Ensure each issue_item has a 'rank_id'
            if (isset($issue_item['rank_id']) && !empty($issue_item['rank_id'])) {
                $prepared_issues_map[$issue_item['rank_id']] = $issue_item;
            } else {
                // Issue item missing 'rank_id' or empty
                // Depending on strictness, you might choose to skip this item or handle it differently
            }
        }

        // If, after processing, the map is empty but raw list wasn't, it means all issues lacked rank_id
        if ( empty($prepared_issues_map) && !empty($raw_issues_list) ) {
            wp_send_json_error( 'Critical error: Issues are missing necessary identifiers (rank_id). Cannot prepare batch.', 500 );
            return;
        }
        
        // Generate a batch ID
        $batch_id = 'rank_batch_' . uniqid();
        
        // Store issues in batch option using the associative map
        $batch_data = array(
            'issue_type' => $issue_type,
            'issues'     => $prepared_issues_map, // Use the associative map
            'decisions'  => array(), // Will store user decisions, keyed by rank_id
            'pending'    => count( $prepared_issues_map ),
            'complete'   => 0,
            'failed'     => 0,
            'ui_logs'    => array(), // For UI logging
            'created_at' => time()
        );
        
        update_option( $batch_id, $batch_data, false ); // 'false' for autoload

        wp_send_json_success( array(
            'batchId' => $batch_id,
            'count'   => count( $prepared_issues_map )
        ) );
    }

    /**
     * Get batch state and next issue to process
     */
    public static function ajax_get_batch_state() {
        
        check_ajax_referer( 'rank_ajax_nonce', 'nonce' );

        if ( ! current_user_can( 'manage_options' ) ) {
            wp_send_json_error( 'Permission denied.', 403 );
        }

        $batch_id = isset( $_POST['batchId'] ) ? sanitize_text_field( $_POST['batchId'] ) : '';
        if ( empty( $batch_id ) ) {
            wp_send_json_error( 'Batch ID not specified.', 400 );
        }

        // Get batch data
        $batch_data = get_option( $batch_id );
        if ( ! $batch_data ) {
            wp_send_json_error( 'Batch not found.', 404 );
        }

        // Create an array to store only new UI logs for this request
        $new_ui_logs = array();

        $next_row = null;
        $credits_exhausted_flag = false; // Flag to indicate if credits ran out during this call
        $api_error_400_halt_flag = false; // Flag for generic 400 API error
        $batch_status_message = '';

        // Now $batch_data['issues'] is an associative array keyed by rank_id (the complex key)
        foreach ( $batch_data['issues'] as $current_row_key => $issue ) {
            // Skip if we already made a decision on this one using the authoritative $current_row_key
            if ( isset( $batch_data['decisions'][$current_row_key] ) ) {
                continue;
            }
            
            // Extract page URL from the issue data before making the AI fix request
            $api_page_full_url = $issue['page']['path_full'] ?? null;
            $wp_page_url = $api_page_full_url ? esc_url($api_page_full_url) : null;
            
            // Check if the URL exists in WordPress
            $url_exists = false;
            if (!empty($wp_page_url)) {
                // First try the standard WordPress function
                $wp_post_id = url_to_postid($wp_page_url);
                
                // If standard function fails, try enhanced method for multilingual sites
                if ($wp_post_id <= 0 && class_exists('Rank_Processor') && method_exists('Rank_Processor', 'enhanced_url_to_postid')) {
                    $wp_post_id = Rank_Processor::enhanced_url_to_postid($wp_page_url);
                }
                
                $url_exists = ($wp_post_id > 0);
                
            }
            
            // Skip this issue if the URL doesn't exist in WordPress
            if (!empty($wp_page_url) && !$url_exists) {
                $log_message = "Skipping URL as it does not exist on our website: " . $wp_page_url;
                $batch_data['ui_logs'][] = $log_message;
                $new_ui_logs[] = $log_message;
                
                // Mark this issue as skipped
                $batch_data['decisions'][$current_row_key] = array(
                    'approved' => false,
                    'error'    => true,
                    'message'  => 'URL does not exist on this WordPress site'
                );
                
                continue; // Skip to the next issue
            }
            
            // Get AI fix for this issue
            $ai_fix = rank_get_ai_fix( $current_row_key ); // domain_identifier is handled within rank_get_ai_fix

            if ( is_wp_error( $ai_fix ) ) {
                $error_code = $ai_fix->get_error_code();
                $error_message = $ai_fix->get_error_message();
                $error_data = $ai_fix->get_error_data();
                $http_status_code = null;

                if ( is_array( $error_data ) && isset( $error_data['status_code'] ) ) {
                    $http_status_code = (int) $error_data['status_code'];
                }
                
                if ( 'out_of_credits' === $error_code ) {
                    // AI credits exhausted
                    // AI credits exhausted
                    $item_name = isset($item_identifier_from_issue) ? basename($item_identifier_from_issue) : 'Unknown';
                    $page_path = isset($wp_page_path_display) ? $wp_page_path_display : 'Unknown page';
                    $log_message = '⚠️ AI Credits Exhausted while processing Image: ' . esc_html($item_name) . ' on /' . esc_html($page_path) . '. Cannot fetch further suggestions.';
                    $batch_data['ui_logs'][] = $log_message;
                    $new_ui_logs[] = $log_message;
                    // Mark current item as unable to process due to no credits
                    $batch_data['decisions'][$current_row_key] = array(
                        'approved'       => false,
                        'error'          => true,
                        'message'        => $error_message, // "AI credits exhausted."
                        'out_of_credits' => true
                    );
                    $credits_exhausted_flag = true;
                    $batch_status_message = 'AI credits exhausted. Processing stopped.';
                    // Do not prepare $next_row, and break the loop as we can't proceed.
                    break;
                } elseif ( $http_status_code === 400 && $error_code === 'api_error' ) {
                    // Handle generic 400 API errors (that are not 'out_of_credits')
                    // API returned HTTP 400 error
                    // API returned HTTP 400 error
                    $item_name = isset($item_identifier_from_issue) ? basename($item_identifier_from_issue) : 'Unknown';
                    $page_path = isset($wp_page_path_display) ? $wp_page_path_display : 'Unknown page';
                    $log_message = '🐞 API Error (400) for Image: ' . esc_html($item_name) . ' on /' . esc_html($page_path) . ': ' . esc_html($error_message) . '. Processing halted.';
                    $batch_data['ui_logs'][] = $log_message;
                    $new_ui_logs[] = $log_message;
                    $batch_data['decisions'][$current_row_key] = array(
                        'approved' => false,
                        'error'    => true,
                        'message'  => 'API Error (400): ' . $error_message
                    );
                    $api_error_400_halt_flag = true;
                    $batch_status_message = 'A critical API error (HTTP 400) occurred. Processing stopped.';
                    break; // Halt batch processing for this request
                } else {
                    // Handle other errors from rank_get_ai_fix (e.g., 5xx, connection issues, non-400 'api_error')
                    // Error getting AI fix
                    // Error getting AI fix
                    $item_name = isset($item_identifier_from_issue) ? basename($item_identifier_from_issue) : 'Unknown';
                    $page_path = isset($wp_page_path_display) ? $wp_page_path_display : 'Unknown page';
                    $log_message = '🐞 Error for Image: ' . esc_html($item_name) . ' on /' . esc_html($page_path) . ': ' . esc_html($error_message);
                    $batch_data['ui_logs'][] = $log_message;
                    $new_ui_logs[] = $log_message;
                    $batch_data['decisions'][$current_row_key] = array(
                        'approved' => false,
                        'error'    => true,
                        'message'  => $error_message
                    );
                    // For other errors, continue to the next item in the batch.
                    // The current item is now "decided" (as an error).
                    continue;
                }
            }
            
            // If $ai_fix is not a WP_Error, it should contain the 'content' key with the solution(s)
            // The structure of $ai_fix (successful response) is assumed to be:
            // { "content": [ { "solution_type": "...", "solution_value": "..." }, ... ] }
            // Or simply { "content": "single solution string" } - adapt as needed.
            
            $ai_solutions_for_current_row = null;
            
            // Process the AI fix content
            if ( isset( $ai_fix['content'] ) ) {
                if ( is_array( $ai_fix['content'] ) ) {
                    // Content is already an array of solutions
                    $ai_solutions_for_current_row = $ai_fix['content'];
                } elseif ( is_string( $ai_fix['content'] ) ) {
                    // If content is a string, wrap it in an array to match structure of multiple solutions
                    $ai_solutions_for_current_row = array(
                        array( 'solution_type' => 'text', 'solution_value' => $ai_fix['content'] )
                    );
                } else {
                    // Unexpected format for 'content' - try to convert to string
                    $content_string = is_object($ai_fix['content']) || is_array($ai_fix['content'])
                        ? json_encode($ai_fix['content'])
                        : (string)$ai_fix['content'];
                    
                    $ai_solutions_for_current_row = array(
                        array( 'solution_type' => 'text', 'solution_value' => $content_string )
                    );
                }
            } else {
                // This shouldn't happen with our updated rank_get_ai_fix function,
                // but let's handle it just in case
                
                // Try to extract any useful text from the response
                $fallback_content = "Image description";
                
                // Check for common fields that might contain useful text
                foreach (['alt_text', 'suggestion', 'text', 'message', 'description'] as $field) {
                    if (isset($ai_fix[$field]) && is_string($ai_fix[$field])) {
                        $fallback_content = $ai_fix[$field];
                        break;
                    }
                }
                
                $ai_solutions_for_current_row = array(
                    array( 'solution_type' => 'text', 'solution_value' => $fallback_content )
                );
                
                // Log this issue but continue processing
                $item_name = isset($item_identifier_from_issue) ? basename($item_identifier_from_issue) : 'Unknown';
                $page_path = isset($wp_page_path_display) ? $wp_page_path_display : 'Unknown page';
                $log_message = '⚠️ Warning for Image: ' . esc_html($item_name) . ' on /' . esc_html($page_path) . ': Using fallback content.';
                $batch_data['ui_logs'][] = $log_message;
                $new_ui_logs[] = $log_message;
            }


            // If we are here, we have a valid $ai_solutions_for_current_row
            // Prepare the $next_row for the UI
            // This logic is similar to what's in ajax_record_decision for finding the next row,
            // but here we are populating the *current* row that was just fetched.

            $ui_url_display = 'N/A';
            $ui_issue_data = [ /* ... populate with page_url, image_url etc. from $issue ... */ ];
            
            // Extract page_url, image_url etc. from $issue (which is $batch_data['issues'][$current_row_key])
            $batch_api_issue_type = $batch_data['issue_type'] ?? null; // The overall type for this batch
            $current_item_api_validator = $issue['validator'] ?? $batch_api_issue_type; // More specific if available

            // Page context from the issue data
            $api_page_path = $issue['page']['path'] ?? null;
            $api_page_full_url = $issue['page']['path_full'] ?? null;
            $wp_page_url = $api_page_full_url ? esc_url($api_page_full_url) : null; // Prefer full URL if available
            $wp_page_path_display = $api_page_path ? esc_html(ltrim($api_page_path, '/')) : 'N/A';
            
            // Extract URL fragments from the API


            $ui_issue_data['page_url'] = $wp_page_url;
            $ui_issue_data['page_path'] = $wp_page_path_display; // Relative path for display

            // Item identifier from the issue data (e.g. image URL, broken link URL)
            $item_identifier_from_issue = $issue['issue_identifier'] ?? null;
            if ($item_identifier_from_issue === $current_item_api_validator && !filter_var($item_identifier_from_issue, FILTER_VALIDATE_URL)) {
                // If issue_identifier is just the validator type (e.g., "missing_meta_description") and not a URL, clear it for item-specific display.
                // The page itself is the item in this case.
                $item_identifier_from_issue = null; 
            }


            // Get the site URL to construct full URLs
            $site_url = site_url();
            
            // Ensure site URL ends with a slash
            if (substr($site_url, -1) !== '/') {
                $site_url .= '/';
            }
            
            // Construct a full URL from the path fragment
            $full_page_url = $wp_page_url;
            if (empty($full_page_url) && !empty($api_page_path)) {
                // Remove leading slash if present
                $path_for_url = ltrim($api_page_path, '/');
                $full_page_url = $site_url . $path_for_url;
            }
            
            switch ($current_item_api_validator) {
                case 'missing_alt_tags':
                    $ui_url_display = 'Image: ' . ($item_identifier_from_issue ? basename($item_identifier_from_issue) : 'Unknown') . ' on <a href="' . esc_url($full_page_url) . '" target="_blank">' . esc_html($wp_page_path_display) . '</a>';
                    $ui_issue_data['image_url'] = $item_identifier_from_issue;
                    $ui_issue_data['page_url'] = $full_page_url; // Store the full URL
                    break;
                case 'missing_meta_description':
                case 'short_meta_description':
                case 'long_meta_description':
                    $desc = '';
                    if ($current_item_api_validator === 'missing_meta_description') $desc = 'Missing meta description';
                    if ($current_item_api_validator === 'short_meta_description') $desc = 'Short meta description';
                    if ($current_item_api_validator === 'long_meta_description') $desc = 'Long meta description';
                    $ui_url_display = '<a href="' . esc_url($full_page_url) . '" target="_blank">' . esc_html($wp_page_path_display) . '</a> - ' . $desc;
                    $ui_issue_data['page_url'] = $full_page_url; // Store the full URL
                    break;
                case 'missing_title_tag':
                case 'short_title_tag':
                case 'long_title_tag':
                    $desc = '';
                    if ($current_item_api_validator === 'missing_title_tag') $desc = 'Missing title';
                    if ($current_item_api_validator === 'short_title_tag') $desc = 'Short title';
                    if ($current_item_api_validator === 'long_title_tag') $desc = 'Long title';
                    $ui_url_display = '<a href="' . esc_url($full_page_url) . '" target="_blank">' . esc_html($wp_page_path_display) . '</a> - ' . $desc;
                    $ui_issue_data['page_url'] = $full_page_url; // Store the full URL
                    break;
                case 'broken_links_external':
                case 'broken_links_internal':
                    $link_type = ($current_item_api_validator === 'broken_links_external') ? 'External' : 'Internal';
                    $ui_url_display = 'Broken ' . $link_type . ' Link: ' . ($item_identifier_from_issue ? esc_html($item_identifier_from_issue) : 'N/A') . ' on <a href="' . esc_url($full_page_url) . '" target="_blank">' . esc_html($wp_page_path_display) . '</a>';
                    if ($item_identifier_from_issue && filter_var($item_identifier_from_issue, FILTER_VALIDATE_URL)) {
                       $ui_issue_data['item_specific_url_generic'] = $item_identifier_from_issue; // The broken URL
                    }
                    $ui_issue_data['page_url'] = $full_page_url; // Store the full URL
                    break;
                default:
                    $page_display_part = '<a href="' . esc_url($full_page_url) . '" target="_blank">' . esc_html($wp_page_path_display) . '</a>';
                    $item_display_part = $item_identifier_from_issue ? esc_html($item_identifier_from_issue) : "Details for " . $current_row_key;
                    $ui_url_display = $page_display_part . ($item_identifier_from_issue ? ': ' . $item_display_part : '');
                    if ($item_identifier_from_issue && filter_var($item_identifier_from_issue, FILTER_VALIDATE_URL)) {
                       $ui_issue_data['item_specific_url_generic'] = $item_identifier_from_issue;
                    }
                    $ui_issue_data['page_url'] = $full_page_url; // Store the full URL
                    break;
            }

            // Prepare the next row data for the frontend
            
            $next_row = array(
                'rowKey'        => $current_row_key,
                'url'           => $ui_url_display,
                'solution'      => $ai_solutions_for_current_row, // This is the array of solutions
                'issue_data'    => $ui_issue_data
            );
            
            // We found an item to process, so break the loop.
            // The decision for this item will be made by the user via ajax_record_decision.
            break;
        } // End foreach loop

        // Save batch data (potentially updated ui_logs and decisions for errors)
        update_option( $batch_id, $batch_data );

        $total_issues = isset($batch_data['issues']) && is_array($batch_data['issues']) ? count($batch_data['issues']) : 0;
        $processed_count = isset($batch_data['decisions']) ? count($batch_data['decisions']) : 0;

        // Fetch current credits information
        $credits_data = rank_get_ai_credits();
        $credits_info = array( 'remaining' => 0, 'total' => 0 );
        
        if ( !is_wp_error( $credits_data ) && is_array( $credits_data ) ) {
            if ( isset( $credits_data['limit'] ) && isset( $credits_data['usage'] ) ) {
                $credits_info['total'] = intval( $credits_data['limit'] );
                $credits_info['remaining'] = max( 0, intval( $credits_data['limit'] ) - intval( $credits_data['usage'] ) );
            }
        }

        $response_payload = array(
            'nextRow'        => $next_row,
            'decisions'      => $batch_data['decisions'],
            'ui_logs'        => $new_ui_logs, // Only send new logs generated during this request
            'totalCount'     => $total_issues,
            'processedCount' => $processed_count,
            'status'         => 'processing', // Default status
            'message'        => $batch_status_message ?: ($next_row ? 'Next item fetched.' : 'No more items to process or batch complete.'),
            'credits'        => $credits_info // Add credits information to the response
        );

        if ( $api_error_400_halt_flag ) {
            $response_payload['status'] = 'api_error_400_halt'; // Specific status for JS
            // $response_payload['message'] is already set by $batch_status_message
        } elseif ( $credits_exhausted_flag ) {
            $response_payload['status'] = 'out_of_credits';
            // $response_payload['message'] is already set by $batch_status_message
        } elseif ( ! $next_row && $processed_count >= $total_issues && $total_issues > 0 ) {
            // All items have decisions (either processed, errored, or skipped by prior logic)
            $response_payload['status'] = 'batch_complete';
            $response_payload['message'] = 'All items in the batch have been processed or encountered an error.';
        } elseif ( ! $next_row && $total_issues === 0 ) {
            $response_payload['status'] = 'batch_empty';
            $response_payload['message'] = 'No issues found in this batch to process.';
        } elseif ( ! $next_row && ! $credits_exhausted_flag && ! $api_error_400_halt_flag ) {
            // This case implies all items have decisions, or some other edge case.
            // If processed_count < total_issues, it means some items were skipped by 'continue'
            // but no new 'next_row' was found, and no halt flags were set.
            // This could happen if all remaining items errored out in a way that used 'continue'.
            $all_items_handled = true;
            if (isset($batch_data['issues']) && is_array($batch_data['issues'])) {
                foreach ($batch_data['issues'] as $key => $issue_data) {
                    if (!isset($batch_data['decisions'][$key])) {
                        $all_items_handled = false;
                        break;
                    }
                }
            }
            
            if ($all_items_handled) {
                 $response_payload['status'] = 'batch_complete';
                 $response_payload['message'] = 'All items in the batch have been processed or encountered an error.';
                 if (empty($batch_data['issues'])) {
                     $response_payload['message'] = 'No issues found in this batch.';
                 }
            }
            // If not all items handled and nextRow is null, it implies an issue or waiting for JS to re-trigger.
            // The JS side should handle this by possibly re-calling if it expects more items.
        }


        
        wp_send_json_success( $response_payload );
    }

    /**
     * Record user decision (approve/decline)
     */
    public static function ajax_record_decision() {
        
        // Tell PHP to stop execution when the client disconnects
        ignore_user_abort(false);
        
        // Set a reasonable timeout for this specific request
        set_time_limit(10);
        
        check_ajax_referer( 'rank_ajax_nonce', 'nonce' );

        if ( ! current_user_can( 'manage_options' ) ) {
            wp_send_json_error( array( 'message' => 'Unauthorized' ), 403 );
            return;
        }

        $batch_id = isset( $_POST['batchId'] ) ? sanitize_key( $_POST['batchId'] ) : null;
        $row_key  = isset( $_POST['rowKey'] ) ? sanitize_text_field( $_POST['rowKey'] ) : null;
        $approved = isset( $_POST['approved'] ) ? filter_var( $_POST['approved'], FILTER_VALIDATE_BOOLEAN ) : false;


        if ( ! $batch_id || ! $row_key ) {
            wp_send_json_error( array( 'message' => 'Missing batch ID or row key.' ), 400 );
            return;
        }

        // Retrieve new POST variables expected from the frontend
        $page_url_from_post = isset($_POST['page_url']) ? esc_url_raw(trim(wp_unslash($_POST['page_url']))) : null;
        $item_url_from_post = isset($_POST['item_url']) ? esc_url_raw(trim(wp_unslash($_POST['item_url']))) : null;
        $solution_from_post = isset($_POST['solution']) ? wp_unslash($_POST['solution']) : null;
        
        // Debug: Log the solution data received
        error_log("RANK DEBUG: Solution data received in record_decision: " . print_r($solution_from_post, true));
        
        // Simple solution extraction
        if (is_string($solution_from_post)) {
            error_log("RANK DEBUG: Solution is a string");
            // Check if it's a serialized array
            if (is_serialized($solution_from_post)) {
                error_log("RANK DEBUG: Solution is serialized, attempting to unserialize");
                $unserialized = maybe_unserialize($solution_from_post);
                if (is_array($unserialized) && isset($unserialized['solution_value'])) {
                    $solution_from_post = $unserialized['solution_value'];
                    error_log("RANK DEBUG: Extracted solution_value from unserialized data: " . $solution_from_post);
                }
            }
            $solution_from_post = trim($solution_from_post);
        } elseif (is_array($solution_from_post)) {
            error_log("RANK DEBUG: Solution is an array with " . count($solution_from_post) . " items");
            // If it's an array with a single object (common case from logs)
            if (count($solution_from_post) === 1 && isset($solution_from_post[0]) && is_array($solution_from_post[0])) {
                error_log("RANK DEBUG: Solution is an array with a single object");
                if (isset($solution_from_post[0]['solution_value'])) {
                    $solution_from_post = $solution_from_post[0]['solution_value'];
                    error_log("RANK DEBUG: Extracted solution_value from array item: " . $solution_from_post);
                }
            }
            // If it's a direct array with solution_value
            elseif (isset($solution_from_post['solution_value'])) {
                $solution_from_post = $solution_from_post['solution_value'];
            }
            // If it's a simple array, use the first element
            elseif (isset($solution_from_post[0])) {
                $solution_from_post = $solution_from_post[0];
            }
            
            // Ensure it's a string and trim it
            if (is_string($solution_from_post)) {
                $solution_from_post = trim($solution_from_post);
            }
        }
        // $api_issue_type_from_post will be derived after fetching $current_issue_data or from POST

        $batch_data = get_option( $batch_id ); 


        if ( ! $batch_data || ! isset( $batch_data['issues'][$row_key] ) ) {
            wp_send_json_error( array( 'message' => 'Batch data or specific issue not found.' ), 404 );
            return;
        }

        // Ensure decisions and ui_logs are initialized as arrays
        if ( ! isset( $batch_data['decisions'] ) || ! is_array( $batch_data['decisions'] ) ) {
            $batch_data['decisions'] = array();
        }
        if ( ! isset( $batch_data['ui_logs'] ) || ! is_array( $batch_data['ui_logs'] ) ) {
            $batch_data['ui_logs'] = array();
        }

        $batch_data['decisions'][$row_key] = $approved; // Store simple approval status
        // For storing the actual solution if edited, the frontend should send it,
        // and it should be stored in $batch_data['decisions'][$row_key]['solution'] if needed for ajax_schedule_tasks
        if ($approved && $solution_from_post !== null) {
            if (!is_array($batch_data['decisions'][$row_key])) { // If it was just a boolean
                 $batch_data['decisions'][$row_key] = ['approved' => true];
            }
            $batch_data['decisions'][$row_key]['approved'] = true;
            $batch_data['decisions'][$row_key]['solution'] = $solution_from_post;
        } elseif (!$approved) {
             $batch_data['decisions'][$row_key] = ['approved' => false];
        }


        $current_issue_data = $batch_data['issues'][$row_key];
        $api_issue_type_from_post = isset($_POST['api_issue_type']) ? sanitize_key($_POST['api_issue_type']) : ($current_issue_data['validator'] ?? $batch_data['issue_type'] ?? null);


        $action_text = $approved ? 'approved' : 'declined';
        
        // Correctly access nested data for logging and task arguments
        $log_identifier_url = $page_url_from_post ?? $item_url_from_post ?? $row_key;
        $actual_issue_type_for_log = $api_issue_type_from_post ?? 'N/A';
        $actual_solution_for_log = $solution_from_post ?? 'N/A (Solution not provided in POST)';
        
        // Ensure solution is a string, trim it, and provide a default if empty after trimming.
        $actual_solution_for_log = is_string($actual_solution_for_log) ? trim($actual_solution_for_log) : 'N/A';
        if (empty($actual_solution_for_log)) {
            $actual_solution_for_log = 'N/A';
        }

        // Create detailed log message for server logs only (not for UI)
        $detailed_log_message = sprintf(
            'Item for %s (%s) %s. Solution: "%s"',
            $log_identifier_url,
            $actual_issue_type_for_log,
            $action_text,
            $actual_solution_for_log
        );
        
        // Log to server error log only, not to UI
        error_log('RANK AJAX (ajax_record_decision): ' . $detailed_log_message);
        
        // No longer adding this message to UI logs
        if ( count( $batch_data['ui_logs'] ) > 500 ) { // Limit log size
            $batch_data['ui_logs'] = array_slice( $batch_data['ui_logs'], 0, 500 );
        }

        if ( $approved ) {
            if ( ! function_exists('as_enqueue_async_action') ) {
                $error_message = 'Error: Action Scheduler function as_enqueue_async_action() not available. Task not scheduled.';
                // Action Scheduler function not available
                $batch_data['ui_logs'][] = $error_message; // Add to UI logs as well
                update_option( $batch_id, $batch_data ); // Save updated batch data
                wp_send_json_error( array( 'message' => $error_message, 'log' => $batch_data['ui_logs'] ) );
                return; // Ensure execution stops
            }

            // Check if the URL exists in WordPress before scheduling the task
            $wp_post_id = 0;
            $url_exists = false;
            
            if (!empty($page_url_from_post)) {
                // First try the standard WordPress function
                $wp_post_id = url_to_postid($page_url_from_post);
                
                // If standard function fails, try enhanced method for multilingual sites
                if ($wp_post_id <= 0 && class_exists('Rank_Processor') && method_exists('Rank_Processor', 'enhanced_url_to_postid')) {
                    $wp_post_id = Rank_Processor::enhanced_url_to_postid($page_url_from_post);
                }
                
                $url_exists = ($wp_post_id > 0);
                
            }
            
            if (!$url_exists) {
                // URL doesn't exist in WordPress, log this and skip scheduling the task
                $log_message = "Skipping URL as it does not exist on our website: " . ($page_url_from_post ?? 'N/A');
                array_unshift($batch_data['ui_logs'], $log_message);
                
                // Save batch data with the new log message
                update_option($batch_id, $batch_data);
                
                // Return success but indicate the URL was skipped
                wp_send_json_success(array(
                    'message'        => '⚠️ URL skipped (not found on site)',
                    'decisions'      => $batch_data['decisions'],
                    'ui_logs'        => $batch_data['ui_logs'],
                    'nextRow'        => null, // Will trigger batch completion
                    'totalCount'     => isset($batch_data['issues']) && is_array($batch_data['issues']) ? count($batch_data['issues']) : 0,
                    'processedCount' => isset($batch_data['decisions']) ? count($batch_data['decisions']) : 0,
                ));
                return;
            }
            
            // Prepare arguments for the scheduled task
            // Use the more reliable data from POST or well-defined $current_issue_data structure
            $task_args = array(
                'page_context_url' => $page_url_from_post, // URL of the page/post
                'item_specific_url'=> $item_url_from_post, // URL of the specific item (e.g., image)
                'api_issue_type'   => $api_issue_type_from_post, // e.g., 'missing_meta_description'
                'solution'         => $solution_from_post,   // The actual solution text
                'row_key'          => $row_key,
                'batch_id'         => $batch_id,
                'wp_post_id'       => $wp_post_id, // Add the post ID we already found
            );


            $action_id = as_enqueue_async_action(
                'rank_process_single_item_hook', // Ensure this hook matches what Rank_Processor expects
                array( $task_args ), // Pass $task_args directly as the single argument in the array
                'rank-ai-seo'
            );

            if ( $action_id ) {
                // Task scheduled successfully
            } else {
                // Failed to schedule task
                
                // We still need to show errors in the UI
                $ui_error_message = sprintf(
                    'Error: Failed to schedule task for item: %s',
                    esc_html( $row_key )
                );
                array_unshift( $batch_data['ui_logs'], $ui_error_message );
            }
        }

        // Save batch data (decisions, logs)
        update_option( $batch_id, $batch_data );

        // Find next unprocessed row for the UI
        $next_row_for_ui = null;
        if (isset($batch_data['issues']) && is_array($batch_data['issues'])) {
            $batch_api_issue_type = $batch_data['issue_type'] ?? null; // Get the overall issue type for the batch

            foreach ( $batch_data['issues'] as $key => $issue_item_data ) { // $key is rank_id, $issue_item_data is the issue's own data
                if ( ! isset( $batch_data['decisions'][$key] ) ) { // Found an unprocessed item
                    
                    $next_ui_url_display = 'N/A'; // For the main display string in the UI
                    $next_ui_issue_data = [
                        'page_path' => null, // For display in UI
                        'page_url'  => null, // Full URL for processing
                        'image_url' => null  // Full image URL if applicable
                    ];

                    // Attempt to get the API Page ID from the issue data.
                    // Common structure might be $issue_item_data['page_id'] or $issue_item_data['page']['id']
                    $api_page_id = $issue_item_data['page_id'] ?? ($issue_item_data['page']['id'] ?? null);
                    $wp_page_url = null;
                    $wp_page_path_display = 'N/A'; // For UI display

                    if ($api_page_id && class_exists('Rank_API_Helper') && method_exists('Rank_API_Helper', 'get_page_details_from_api_id')) {
                        $page_details = Rank_API_Helper::get_page_details_from_api_id($api_page_id);
                        if ($page_details && !empty($page_details['url'])) {
                            $wp_page_url = $page_details['url'];
                            $parsed_url = parse_url($wp_page_url);
                            $wp_page_path_display = $parsed_url['path'] ?? $wp_page_url; // Use path for display, or full URL as fallback
                        } else {
                            // Could not get page details from API ID
                        }
                    } else {
                         // Missing API Page ID or Rank_API_Helper
                    }
                    
                    $next_ui_issue_data['page_url'] = $wp_page_url; // This is the crucial full URL
                    $next_ui_issue_data['page_path'] = $wp_page_path_display; // For UI text

                    // Get item identifier (e.g., image URL for alt tags, or specific element identifier)
                    $item_identifier_from_issue = $issue_item_data['issue_identifier'] ?? ($issue_item_data['parameters']['url'] ?? null);

                    $ai_solutions_for_next_row = $issue_item_data['solution'] ?? ['Solution not available'];
                    if (is_string($ai_solutions_for_next_row)) { // Ensure it's an array for the UI
                        $ai_solutions_for_next_row = [$ai_solutions_for_next_row];
                    }

                    // Construct 'url' (display string) and 'issue_data' based on the batch's issue type
                    if ($batch_api_issue_type === 'missing_alt_tags') {
                        $next_ui_url_display = ($wp_page_path_display !== 'N/A' ? esc_html($wp_page_path_display) . ' ' : '') . 'Image: ' . esc_html($item_identifier_from_issue ?? 'N/A');
                        $next_ui_issue_data['image_url'] = $item_identifier_from_issue; // This is the item_url for alt tags
                    } elseif ($batch_api_issue_type === 'missing_meta_description') {
                        $next_ui_url_display = (esc_html($wp_page_path_display) ?? 'Page') . ' Missing meta description';
                        // page_url is already set in $next_ui_issue_data
                    } elseif ($batch_api_issue_type === 'missing_title_tag') {
                        $next_ui_url_display = (esc_html($wp_page_path_display) ?? 'Page') . ' Missing title tag';
                        // page_url is already set in $next_ui_issue_data
                    } else {
                        // Generic display for other types
                        $page_display_part = $wp_page_path_display !== 'N/A' ? esc_html($wp_page_path_display) : 'Item';
                        $item_display_part = $item_identifier_from_issue ? esc_html($item_identifier_from_issue) : "Details for " . $key;
                        $next_ui_url_display = $page_display_part . ($item_identifier_from_issue ? ': ' . $item_display_part : '');
                        // If item_identifier is a URL and not an image type, it might be a generic item_url
                        if ($item_identifier_from_issue && filter_var($item_identifier_from_issue, FILTER_VALIDATE_URL) && $batch_api_issue_type !== 'missing_alt_tags') {
                           $next_ui_issue_data['item_specific_url_generic'] = $item_identifier_from_issue;
                        }
                    }

                    $next_row_for_ui = array(
                        'rowKey'        => $key,                         // The unique rank_id
                        'url'           => $next_ui_url_display,         // For UI display string
                        'solution'      => $ai_solutions_for_next_row,   // Array of solutions
                        'issue_data'    => $next_ui_issue_data           // Contains page_url, image_url etc. for JS
                    );
                    break; // Found the next row, exit loop
                }
            }
        }
        
        $total_issues = isset($batch_data['issues']) && is_array($batch_data['issues']) ? count($batch_data['issues']) : 0;
        $processed_count = isset($batch_data['decisions']) ? count($batch_data['decisions']) : 0;

        // Fetch current credits information
        $credits_data = rank_get_ai_credits();
        $credits_info = array( 'remaining' => 0, 'total' => 0 );
        
        if ( !is_wp_error( $credits_data ) && is_array( $credits_data ) ) {
            if ( isset( $credits_data['limit'] ) && isset( $credits_data['usage'] ) ) {
                $credits_info['total'] = intval( $credits_data['limit'] );
                $credits_info['remaining'] = max( 0, intval( $credits_data['limit'] ) - intval( $credits_data['usage'] ) );
            }
        }

        wp_send_json_success( array(
            'message'        => $approved ? '✅ Fixed' : '❌ Fix declined.',
            'decisions'      => $batch_data['decisions'],
            'ui_logs'        => $batch_data['ui_logs'],
            'nextRow'        => $next_row_for_ui, // Send the fully prepared next row for UI
            'totalCount'     => $total_issues,
            'processedCount' => $processed_count,
            'credits'        => $credits_info // Add credits information to the response
        ) );
        
    }

    /**
     * AJAX handler to fetch health issues overview data.
     * This data is used to populate the initial "Fix health problems" tab.
     */
    public static function ajax_get_health_issues_overview() {
        // Tell PHP to stop execution when the client disconnects
        ignore_user_abort(false);
        
        // Set a reasonable timeout for this specific request
        set_time_limit(30);
        
        // Verify nonce
        if (!isset($_POST['nonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['nonce'])), 'rank_ajax_nonce')) {
            wp_send_json_error(array('message' => 'Nonce verification failed.'), 403);
            return;
        }
        
        // Fetch health issue counts with fix status (using sampling)
        $issues_data = rank_get_health_issue_counts_with_fix_status();
        
        // Fetch AI credits information
        $credits_data = rank_get_ai_credits();
        
        // Check for errors
        if (is_wp_error($issues_data)) {
            // Error fetching issues data
            wp_send_json_error(array('message' => 'Error fetching health issues data: ' . $issues_data->get_error_message()), 500);
            return;
        }
        
        if (is_wp_error($credits_data)) {
            // Error fetching credits data
            wp_send_json_error(array('message' => 'Error fetching AI credits data: ' . $credits_data->get_error_message()), 500);
            return;
        }
        
        // Ensure data is in expected array format
        if (!is_array($issues_data)) {
            $issues_data = array();
        }
        
        if (!is_array($credits_data)) {
            $credits_data = array('remaining' => 0, 'total' => 0, 'limit' => 0, 'usage' => 0);
        }
        
        $response_data = array(
            'issues' => $issues_data,
            'credits' => $credits_data
        );
        
        wp_send_json_success($response_data);
    }

    /**
     * AJAX handler to clear pending decisions for a batch.
     * This is called when a user clicks the Cancel button to reset the state.
     */
    public static function ajax_clear_pending_decisions() {
        // Tell PHP to stop execution when the client disconnects
        ignore_user_abort(false);
        
        // Set a reasonable timeout for this specific request
        set_time_limit(10);
        
        check_ajax_referer( 'rank_ajax_nonce', 'nonce' );

        if ( ! current_user_can( 'manage_options' ) ) {
            wp_send_json_error( array( 'message' => 'Permission denied.' ), 403 );
        }

        if ( ! isset( $_POST['batchId'] ) || empty( $_POST['batchId'] ) ) {
            wp_send_json_error( array( 'message' => 'Missing batch ID.' ), 400 );
        }

        $batch_id = sanitize_key( $_POST['batchId'] );
        
        if ( strpos($batch_id, 'rank_batch_') !== 0 ) {
            wp_send_json_error( array( 'message' => 'Invalid batch ID format.' ), 400 );
        }
        
        // Get the batch data
        $batch_data = get_option( $batch_id );
        
        if ( ! $batch_data ) {
            // If batch doesn't exist, nothing to clear
            wp_send_json_success( array( 'message' => 'No batch data found to clear.' ) );
            return;
        }
        
        // Reset the batch state
        if (isset($batch_data['decisions'])) {
            // Clear all decisions that haven't been processed yet
            $batch_data['decisions'] = array();
        }
        
        // Reset counters
        $batch_data['pending'] = isset($batch_data['issues']) ? count($batch_data['issues']) : 0;
        $batch_data['complete'] = 0;
        $batch_data['failed'] = 0;
        
        // Add a log entry
        if ( isset( $batch_data['ui_logs'] ) && is_array( $batch_data['ui_logs'] ) ) {
            $batch_data['ui_logs'][] = 'User canceled processing and reset all pending decisions.';
        } else {
            $batch_data['ui_logs'] = array( 'User canceled processing and reset all pending decisions.' );
        }
        
        // Cancel any scheduled tasks for this batch
        if (function_exists('as_unschedule_all_actions')) {
            $action_group = 'rank-tasks-' . $batch_id;
            as_unschedule_all_actions('rank_process_single_item_hook', array(), $action_group);
            
            if (isset($batch_data['ui_logs']) && is_array($batch_data['ui_logs'])) {
                $batch_data['ui_logs'][] = 'All scheduled tasks for this batch have been canceled.';
            }
        }
        
        // Save the updated batch data
        update_option( $batch_id, $batch_data );
        
        // Log the action
        // User cleared pending decisions for batch
        
        wp_send_json_success( array(
            'message' => 'Pending decisions cleared successfully. All processing has been reset.',
            'batch_data' => $batch_data // Return the updated batch data for the frontend
        ) );
    }

    /**
     * AJAX handler for updating fix values
     */
    public static function ajax_update_fix_value() {
        // Tell PHP to stop execution when the client disconnects
        ignore_user_abort(false);
        
        // Set a reasonable timeout for this specific request
        set_time_limit(10);
        
        check_ajax_referer('rank_ajax_nonce', 'nonce');
        
        if (!current_user_can('manage_options')) {
            wp_send_json_error('Permission denied.', 403);
        }
        
        // Get and sanitize parameters
        $category = isset($_POST['category']) ? sanitize_key($_POST['category']) : '';
        $value = isset($_POST['value']) ? sanitize_text_field($_POST['value']) : '';
        $is_option = isset($_POST['is_option']) && $_POST['is_option'] === 'true';
        
        if (empty($category) || $value === '') {
            wp_send_json_error('Missing required parameters', 400);
        }
        
        // Handle different types of fixes
        if ($is_option) {
            // Handle option-based fixes (broken links)
            $old_url = isset($_POST['old_url']) ? sanitize_text_field($_POST['old_url']) : '';
            if (empty($old_url)) {
                wp_send_json_error('Missing old URL parameter', 400);
            }
            
            $option_name = '';
            if ($category === 'broken_links_internal') {
                $option_name = 'rank_internal_url_replacements';
            } elseif ($category === 'broken_links_external') {
                $option_name = 'rank_external_url_replacements';
            }
            
            if (empty($option_name)) {
                wp_send_json_error('Invalid category for option-based fix', 400);
            }
            
            // Get current replacements
            $replacements = get_option($option_name, []);
            
            // Update the value
            if (is_array($replacements) && array_key_exists($old_url, $replacements)) {
                if (is_array($replacements[$old_url]) && isset($replacements[$old_url]['new_url'])) {
                    $replacements[$old_url]['new_url'] = $value;
                } else {
                    $replacements[$old_url] = $value;
                }
                
                update_option($option_name, $replacements);
                wp_send_json_success();
            } else {
                wp_send_json_error('Old URL not found in replacements', 404);
            }
        } else {
            // Handle database-based fixes
            $post_id = isset($_POST['post_id']) ? absint($_POST['post_id']) : 0;
            $object_id = isset($_POST['object_id']) ? absint($_POST['object_id']) : 0;
            
            if ($post_id <= 0) {
                wp_send_json_error('Invalid post ID', 400);
            }
            
            // Map category to override_type
            $override_type = '';
            switch ($category) {
                case 'missing_meta_description':
                case 'short_meta_description':
                case 'long_meta_description':
                    $override_type = 'meta_description';
                    break;
                case 'missing_title_tag':
                case 'short_title_tag':
                case 'long_title_tag':
                    $override_type = 'title';
                    break;
                case 'missing_alt_tags':
                    $override_type = 'image_alt';
                    break;
                default:
                    wp_send_json_error('Invalid category for database-based fix', 400);
            }
            
            // Use the existing update_seo_data method
            $result = Rank_Processor::update_seo_data($post_id, $object_id, $override_type, $value, $category);
            
            if ($result) {
                wp_send_json_success();
            } else {
                wp_send_json_error('Failed to update database value', 500);
            }
        }
    }
}