<?php
/**
 * Handles frontend filtering for SEO data overrides (title, meta description, alt tags, content URLs).
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

class Rank_Frontend_Filters {

    // Store fetched URL rules to avoid repeated lookups per request
    private static $url_replacement_rules = null;

    /**
     * Initialize frontend filter hooks.
     */
    public static function init() {
        // Image Alt Tags
        add_filter( 'wp_get_attachment_image_attributes', array( __CLASS__, 'filter_attachment_alt' ), 10, 2 );

        // Title Tags
        add_filter( 'pre_get_document_title', array( __CLASS__, 'filter_fallback_title' ), 5, 1 ); // Fallback
        add_filter( 'wpseo_title', array( __CLASS__, 'filter_seo_plugin_title' ), 15, 1 ); // Yoast
        add_filter( 'rank_math/frontend/title', array( __CLASS__, 'filter_seo_plugin_title' ), 15, 1 ); // Rank Math

        // Meta Descriptions
        add_filter( 'wpseo_metadesc', array( __CLASS__, 'filter_meta_description' ), 10, 1 ); // Yoast
        add_filter( 'rank_math/frontend/description', array( __CLASS__, 'filter_meta_description' ), 10, 1 ); // Rank Math
        add_action( 'wp_head', array( __CLASS__, 'add_meta_description_tag_fallback' ), 5 ); // Fallback output

        // --- New Filter for Content URL Replacement ---
        add_filter( 'the_content', array( __CLASS__, 'filter_content_urls' ), 10, 1 ); // Priority 10 is standard
    }

    /**
     * Get override value from the database.
     *
     * @param int    $post_id       Post ID context.
     * @param int    $object_id     Specific object ID (e.g., attachment ID), 0 for post-level.
     * @param string $override_type The type of override ('title', 'meta_description', 'image_alt').
     * @return string|null The override value or null if not found.
     */
    private static function get_override_value( $post_id, $object_id, $override_type ) {
        global $wpdb;

        // Ensure RANK_TABLE_NAME is defined before using it
        if ( ! defined('RANK_TABLE_NAME') ) {
             error_log('RANK Plugin Error: RANK_TABLE_NAME constant not defined in Rank_Frontend_Filters.');
             return null;
        }
        $table_name = $wpdb->prefix . RANK_TABLE_NAME;

        $sql = $wpdb->prepare(
            "SELECT override_value FROM $table_name WHERE post_id = %d AND object_id = %d AND override_type = %s ORDER BY override_id DESC LIMIT 1",
            absint( $post_id ),
            absint( $object_id ),
            sanitize_key( $override_type )
        );
        $override = $wpdb->get_var( $sql );

        return $override; // Returns null if no row found
    }

    /**
     * Fetch and cache URL replacement rules (using WP Options method).
     *
     * @return array Associative array where key is 'old URL' and value is 'new URL'.
     */
    private static function get_url_replacement_rules() {
        if ( self::$url_replacement_rules === null ) {
            // Fetch from WP options, default to empty array if not set
            self::$url_replacement_rules = get_option( 'rank_url_replacements', [] );

            // Basic validation: ensure it's an array
            if ( ! is_array( self::$url_replacement_rules ) ) {
                self::$url_replacement_rules = [];
            }
        }
        return self::$url_replacement_rules;
    }

    /**
     * Filter post content to replace specific URLs.
     *
     * @param string $content The original post content.
     * @return string The modified post content.
     */
    public static function filter_content_urls( $content ) {
        // Only run replacements on single posts/pages/products etc.
        // and only if we are in the main loop to avoid modifying content in widgets, etc.
        if ( ! is_singular() || ! in_the_loop() || ! is_main_query() ) {
            return $content;
        }

        $rules = self::get_url_replacement_rules();

        if ( empty( $rules ) ) {
            return $content; // No rules defined, nothing to do.
        }

        // Extract keys (old URLs) and values (new URLs) from the associative array
        $old_urls = array_keys( $rules );
        $new_urls = array_values( $rules );

        // Perform case-sensitive replacement. Use str_ireplace for case-insensitive.
        $updated_content = str_replace( $old_urls, $new_urls, $content );

        return $updated_content;
    }

    /**
     * Filter image attributes to add alt text override if empty.
     *
     * @param array $attr Attributes for the image tag.
     * @param WP_Post $attachment Attachment object.
     * @return array Modified attributes.
     */
    public static function filter_attachment_alt( $attr, $attachment ) {
        if ( empty( $attr['alt'] ) && is_object($attachment) && isset($attachment->ID) ) {
            $attachment_id = absint( $attachment->ID );
            // For alt tags, the primary identifier is the attachment ID (object_id).
            // We still need a post_id context, but it might be 0 if the image isn't directly tied to the current singular post.
            // Let's try getting the current post ID if available, otherwise use 0.
            $current_post_id = is_singular() ? get_queried_object_id() : 0;

            // Query using object_id = attachment_id and override_type = 'image_alt'.
            // We check both post_id = $current_post_id AND post_id = 0 to cover overrides
            // saved with or without specific page context. Prioritize specific page context.
            $alt_override = self::get_override_value( $current_post_id, $attachment_id, 'image_alt' );
            if ($alt_override === null && $current_post_id !== 0) {
                 $alt_override = self::get_override_value( 0, $attachment_id, 'image_alt' );
            }


            if ( $alt_override !== null ) {
                $attr['alt'] = $alt_override; // Value already sanitized on save
            }
        }
        return $attr;
    }

    /**
     * Fallback function to filter the document title if major SEO plugins are inactive.
     *
     * @param string $title The original title.
     * @return string The original or overridden title.
     */
    public static function filter_fallback_title( $title ) {
        if ( defined('WPSEO_VERSION') || defined('RANK_MATH_VERSION') ) {
            return $title;
        }

        if ( is_singular() ) {
            $post_id = get_queried_object_id();
            if ( $post_id > 0 ) {
                $title_override = self::get_override_value( $post_id, 0, 'title' );
                if ( $title_override !== null ) {
                    return esc_html( $title_override );
                }
            }
        }
        return $title;
    }

    /**
     * Callback function to potentially override the title generated by SEO plugins.
     *
     * @param string $title The original title generated by the SEO plugin.
     * @return string The original or overridden title.
     */
    public static function filter_seo_plugin_title( $title ) {
        if ( is_singular() ) {
            $post_id = get_queried_object_id();
            if ( $post_id > 0 ) {
                $title_override = self::get_override_value( $post_id, 0, 'title' );
                if ( $title_override !== null ) {
                    return esc_html( $title_override );
                }
            }
        }
        return $title;
    }

    /**
     * Callback function to potentially override the meta description generated by SEO plugins.
     *
     * @param string $description The original description.
     * @return string The original or overridden description.
     */
    public static function filter_meta_description( $description ) {
        if ( is_singular() ) {
            $post_id = get_queried_object_id();
            if ( $post_id > 0 ) {
                $desc_override = self::get_override_value( $post_id, 0, 'meta_description' );
                if ( $desc_override !== null ) {
                    // Use the override value directly (sanitized on save)
                    return $desc_override;
                }
            }
        }
        return $description;
    }

    /**
     * Fallback function to output the meta description tag directly if no known SEO plugin is active.
     */
    public static function add_meta_description_tag_fallback() {
        if ( defined('WPSEO_VERSION') || defined('RANK_MATH_VERSION') || defined('AIOSEO_VERSION') || defined('SEOPRESS_VERSION') ) {
            return;
        }
        if ( ! is_singular() ) {
            return;
        }
        $post_id = get_queried_object_id();
        if ( ! $post_id ) {
            return;
        }

        $desc_override = self::get_override_value( $post_id, 0, 'meta_description' );

        if ( $desc_override !== null ) {
            echo '<meta name="description" content="' . esc_attr( $desc_override ) . '" />' . "\n";
        }
    }
} 