<?php
/**
 * Handles processing of scheduled SEO update tasks via Action Scheduler.
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

class Rank_Processor {

    /**
     * Initialize processor hooks.
     */
    public static function init() {
        // Hook for tasks from manual approval (passes 1 array arg)
        add_action( 'rank_process_single_item_hook', array( __CLASS__, 'run_scheduled_task' ), 10, 1 );

        // Hook for tasks from bulk scheduling (passes 5 distinct args - assumed from original code)
        // This ensures 'rank_process_single_url' hook, if used by another process for bulk actions,
        // is also handled by run_scheduled_task with its expected 5 arguments.
        add_action( 'rank_process_single_url', array( __CLASS__, 'run_scheduled_task' ), 10, 5 );
    }

    /**
     * Runs the scheduled task.
     * This method can be called by Action Scheduler with different argument structures
     * based on the hook that scheduled it.
     *
     * @param mixed $arg1 First argument. Can be an array of task details or the first of 5 distinct args.
     * @param mixed $arg2 Optional. Second of 5 distinct args.
     * @param mixed $arg3 Optional. Third of 5 distinct args.
     * @param mixed $arg4 Optional. Fourth of 5 distinct args.
     * @param mixed $arg5 Optional. Fifth of 5 distinct args.
     */
    public static function run_scheduled_task( $arg1, $arg2 = null, $arg3 = null, $arg4 = null, $arg5 = null ) {
        try {
            // Always log for debugging
            // Removed verbose debug logging for production
            $task_args = array();
            $is_five_arg_payload = false;
            $is_array_payload = false;

            // Determine argument structure
            if ( ! is_array( $arg1 ) && $arg5 !== null ) { // Check for 5 distinct arguments
                $is_five_arg_payload = true;
                // Argument mapping:
                // $arg1 = original_url (page context)
                // $arg2 = item_specific_url (e.g., image URL, or broken link URL)
                // $arg3 = api_issue_type
                // $arg4 = solution
                // $arg5 = batch_id
                $task_args = array(
                    'batch_id'         => $arg5,
                    'page_context_url' => $arg1, 
                    'item_specific_url'=> $arg2,
                    'api_issue_type'   => $arg3,
                    'solution'         => $arg4,
                    'original_url'     => $arg1, // Keep for backward compatibility if any logic relies on 'original_url' explicitly
                    'row_key'          => 'from_rank_process_single_url_' . md5(print_r(func_get_args(), true)),
                );
            }
            elseif ( is_array( $arg1 ) && isset( $arg1['row_key'] ) && isset( $arg1['api_issue_type'] ) && isset($arg1['page_context_url']) ) {
                $task_args = $arg1;
                $is_array_payload = true;
            }
            elseif ( func_num_args() === 5 && !is_array($arg1) ) {
                $task_args = array(
                    'page_id_raw'    => $arg2,
                    'object_id_raw'  => $arg5,
                    'api_issue_type' => $arg3,
                    'solution'       => $arg4,
                    'original_url'   => $arg1,
                    'row_key'        => 'from_rank_process_single_url_' . md5(print_r(func_get_args(), true)),
                );
            }
            elseif ( is_array( $arg1 ) && isset( $arg1['batch_id'] ) && isset( $arg1['issue_type'] ) ) {
                $task_args = $arg1;
                if (isset($task_args['issue_type'])) $is_array_payload = true;
            }
            else {
                error_log("Rank_Processor: run_scheduled_task called with unexpected arguments.");
                return;
            }

            $batch_id         = isset($task_args['batch_id']) ? sanitize_key($task_args['batch_id']) : null;
            $page_context_url = isset( $task_args['page_context_url'] ) ? esc_url_raw( $task_args['page_context_url'] ) : null;
            $item_specific_url = isset( $task_args['item_specific_url'] ) ? esc_url_raw( $task_args['item_specific_url'] ) : null;
            $api_issue_type   = isset( $task_args['api_issue_type'] ) ? $task_args['api_issue_type'] : ($task_args['issue_type'] ?? null);
            $override_value   = isset( $task_args['solution'] ) ? $task_args['solution'] : null;
            
            // Special logging for short_meta_description
            
            // Simple solution extraction
            if (is_string($override_value)) {
                // Check if it's a serialized array
                if (is_serialized($override_value)) {
                    $unserialized = maybe_unserialize($override_value);
                    if (is_array($unserialized) && isset($unserialized['solution_value'])) {
                        $override_value = $unserialized['solution_value'];
                    }
                }
            } elseif (is_array($override_value)) {
                // If it's an array with a single object (common case from logs)
                if (count($override_value) === 1 && isset($override_value[0]) && is_array($override_value[0])) {
                    if (isset($override_value[0]['solution_value'])) {
                        $override_value = $override_value[0]['solution_value'];
                    }
                }
                // If it's a direct array with solution_value
                elseif (isset($override_value['solution_value'])) {
                    $override_value = $override_value['solution_value'];
                }
                // If it's a simple array, use the first element
                elseif (isset($override_value[0])) {
                    $override_value = $override_value[0];
                }
            }
            $row_key          = isset( $task_args['row_key'] ) ? $task_args['row_key'] : 'unknown_row_key_' . uniqid();

            // Ensure critical task_args are present
            if (empty($api_issue_type)) {
                error_log("Rank_Processor: api_issue_type is missing. Row Key: {$row_key}. Task Args: " . print_r($task_args, true));
                if ($batch_id) {
                    self::add_log_to_transient( $batch_id, "Processor Error: Critical task data (api_issue_type) missing.", $row_key, 'error' );
                }
                return; // Cannot proceed
            }

            if (empty($page_context_url) && isset($task_args['original_url'])) {
                $page_context_url = esc_url_raw($task_args['original_url']);
            }
            // For missing_alt_tags, item_specific_url is the image URL.
            // For broken links, item_specific_url should be the broken URL itself.
            // The original logic for item_specific_url for missing_alt_tags might need to be conditional.
            if (empty($item_specific_url)) {
                if (isset($task_args['original_url']) && $api_issue_type === 'missing_alt_tags') {
                    if ($page_context_url !== $task_args['original_url'] || !isset($task_args['page_context_url'])) {
                        $item_specific_url = esc_url_raw($task_args['original_url']);
                    }
                }
                // For broken links, try to extract the broken URL from the row_key if it's a base64 encoded JSON
                elseif (($api_issue_type === 'broken_links_external' || $api_issue_type === 'broken_links_internal') && !empty($row_key)) {
                    // Try to decode the row_key if it looks like base64
                    if (preg_match('/^[a-zA-Z0-9\/\r\n+]*={0,2}$/', $row_key)) {
                        $decoded = base64_decode($row_key, true);
                        if ($decoded !== false && is_string($decoded)) {
                            $json_data = json_decode($decoded, true);
                            if (is_array($json_data) && isset($json_data['issue_identifier'])) {
                                // The issue_identifier might be the broken URL
                                $item_specific_url = esc_url_raw($json_data['issue_identifier']);
                            }
                        }
                    }
                }
            }

            $db_override_type = '';
            $needs_database_update = true; // Flag to determine if update_seo_data should be called

            switch ( $api_issue_type ) {
                case 'missing_meta_description':
                case 'short_meta_description':
                case 'long_meta_description':
                    $db_override_type = 'meta_description';
                    break;
                case 'missing_title_tag':
                case 'short_title_tag':
                case 'long_title_tag':
                    $db_override_type = 'title';
                    break;
                case 'missing_alt_tags':
                    $db_override_type = 'image_alt';
                    break;
                case 'broken_links_external':
                case 'broken_links_internal':
                    $needs_database_update = false; // These will be handled by storing suggestions
                    
                    // If we still don't have a broken URL but we have a row_key, try to use the row_key itself
                    // as a fallback since it might contain the broken URL in some format
                    if (empty($item_specific_url) && !empty($row_key) && strpos($row_key, 'http') !== false) {
                        // Extract URL-like string from row_key as a last resort
                        if (preg_match('/https?:\/\/[^\s\)"\']+/', $row_key, $matches)) {
                            $item_specific_url = esc_url_raw($matches[0]);
                        }
                    }
                    
                    if (!empty($page_context_url) && isset($override_value)) {
                        // Store the suggestion even if item_specific_url is empty - we'll use a placeholder
                        $broken_url = !empty($item_specific_url) ? $item_specific_url : '[unknown broken URL]';
                        
                        $suggestion_message = sprintf(
                            "Fix for %s: On page '%s', change link from '%s' to '%s'.",
                            esc_html($api_issue_type),
                            esc_html($page_context_url),
                            esc_html($broken_url), // This is the broken URL or placeholder
                            esc_html($override_value)    // This is the suggested new URL
                        );
                        
                        // Get the appropriate option name
                        $option_name = ($api_issue_type === 'broken_links_internal')
                            ? 'rank_internal_url_replacements'
                            : 'rank_external_url_replacements';
                        
                        // Get current replacements
                        $replacements = get_option($option_name, []);
                        
                        // Use row_key as the key if item_specific_url is empty
                        $key_to_use = !empty($item_specific_url) ? $item_specific_url : 'row_key_' . md5($row_key);
                        
                        // Add or update the replacement
                        $replacements[$key_to_use] = [
                            'new_url' => $override_value,
                            'context' => $page_context_url,
                            'row_key' => $row_key
                        ];
                        
                        // Save the updated replacements
                        update_option($option_name, $replacements);
                        
                        // Clear the health issues cache to ensure fresh data
                        if (function_exists('rank_clear_health_issues_cache')) {
                            rank_clear_health_issues_cache();
                        }
                        
                        if ($batch_id) {
                            self::add_log_to_transient( $batch_id, $suggestion_message, $row_key, 'suggestion' );
                        }
                    } else {
                        $error_message = "[RANK PROCESSOR ERROR] Missing data for {$api_issue_type} suggestion. PageURL: '{$page_context_url}', BrokenURL: '{$item_specific_url}', SuggestedFix: '{$override_value}'. (Row: {$row_key}, Batch: {$batch_id})";
                        error_log($error_message);
                        if ($batch_id) {
                            self::add_log_to_transient( $batch_id, "Error: Missing data for {$api_issue_type} suggestion.", $row_key, 'error' );
                        }
                    }
                    break;
                default:
                    $log_message = "[RANK PROCESSOR ERROR] Unknown API issue type: '{$api_issue_type}' for Row Key {$row_key}. Cannot map to DB override type.";
                    error_log( $log_message );
                    if ($batch_id) {
                        self::add_log_to_transient( $batch_id, "Processor Error: Unknown API issue type '{$api_issue_type}'.", $row_key, 'error' );
                    }
                    return; // Stop processing if type is unknown
            }

            if ( ! $needs_database_update ) {
                // For types like broken links, the action (recording suggestion) is done.
                // Log success for this item if no errors occurred during suggestion recording.
                // error_log("Rank_Processor: Successfully recorded suggestion for row_key '{$row_key}'.");
                // The main try-catch will handle overall success/failure logging for the task.
                // No specific "return" here, allow to proceed to batch completion checks etc. if any.
                // If add_log_to_transient already indicates success/error, that might be sufficient.
                // For now, we assume the task is "handled" if $needs_database_update is false.
            } else {
                // Proceed with database update logic for meta, title, alt tags
                // Check if we already have a post ID from the AJAX handler
                $wp_post_id = isset($task_args['wp_post_id']) ? intval($task_args['wp_post_id']) : 0;
                
                // If we don't have a post ID yet, try to get it from the URL
                if ($wp_post_id <= 0 && !empty($page_context_url)) {
                    // First try the standard WordPress function
                    $wp_post_id = url_to_postid($page_context_url);
                    
                    // If standard function fails, try our enhanced method for multilingual sites
                    if ($wp_post_id <= 0) {
                        $wp_post_id = self::enhanced_url_to_postid($page_context_url);
                        if ($wp_post_id > 0) {
                        }
                    }
                    
                    if ($wp_post_id > 0) {
                    } else {
                        error_log("Rank_Processor: Failed to convert URL to post_id. Result: $wp_post_id");
                    }
                } else if ($wp_post_id > 0) {
                } else {
                    error_log("Rank_Processor: Missing page_context_url. Cannot determine post_id.");
                }

                if ( ! $wp_post_id ) {
                    error_log("Rank_Processor: Skipping task due to missing WordPress Post ID.");
                    if ($batch_id) {
                        self::add_log_to_transient( $batch_id, "Processor Error: Could not find WP Post ID for URL " . esc_html($page_context_url), $row_key, 'error' );
                    }
                    return;
                }

                $object_id_to_use = 0; // Default for non-image types or if image not found

                if ( $db_override_type === 'image_alt' ) {
                    if ( ! empty( $item_specific_url ) ) {
                        $attachment_id = self::get_attachment_id_from_url( $item_specific_url );
                        if ( $attachment_id > 0 ) {
                            $object_id_to_use = $attachment_id;
                        } else {
                            error_log("Rank_Processor: Could not find attachment ID for image. Alt tag will not be applied.");
                            if ($batch_id) {
                                self::add_log_to_transient( $batch_id, "Processor Error: No Attachment ID for image " . esc_html(basename($item_specific_url)), $row_key, 'error' );
                            }
                            return; // Stop processing this item if attachment ID is crucial and not found
                        }
                    } else {
                        error_log("Rank_Processor: Missing item_specific_url for image_alt.");
                         if ($batch_id) {
                            self::add_log_to_transient( $batch_id, "Processor Error: Missing image URL for alt tag.", $row_key, 'error' );
                        }
                        return; // Stop processing if item_specific_url is missing for alt tag
                    }
                } else {
                    // For non-image issues like meta_description, title, object_id is 0.
                    $object_id_to_use = 0;
                }

                // Final check for override_value before DB operation
                if ( $override_value === null ) {
                    error_log("Rank_Processor: Override value is null. Skipping update.");
                    if ($batch_id) {
                        self::add_log_to_transient( $batch_id, "Processor Error: Solution (override value) is missing for {$db_override_type}.", $row_key, 'error' );
                    }
                    return; 
                }

                $update_success = self::update_seo_data( $wp_post_id, $object_id_to_use, $db_override_type, $override_value, $api_issue_type, $batch_id, $row_key );

                if ( $update_success ) {
                    if ($batch_id) {
                        $success_log_msg = ucfirst($db_override_type) . " updated successfully.";
                        if ($db_override_type === 'image_alt' && $item_specific_url) {
                             $success_log_msg = "Alt tag for image '" . basename($item_specific_url) . "' updated.";
                        }
                        self::add_log_to_transient( $batch_id, $success_log_msg, $row_key, 'success' );
                    }
                } else {
                    // add_log_to_transient is called within update_seo_data on failure
                }
            } // End of if ($needs_database_update) else block

        } catch (Throwable $e) { // Throwable catches Errors and Exceptions in PHP 7+
            error_log("Rank_Processor: CRITICAL ERROR in run_scheduled_task. Error: " . $e->getMessage());
            if (isset($batch_id) && $batch_id && isset($row_key)) { // Ensure batch_id and row_key are available
                 self::add_log_to_transient( $batch_id, "CRITICAL Processor Error: " . $e->getMessage(), $row_key, 'critical' );
            }
            throw $e; // Re-throw to ensure Action Scheduler marks it as failed and handles retry logic if applicable.
        }
    }

    /**
     * Updates or inserts SEO override data into the custom table.
     * Uses $wpdb->replace for efficiency.
     *
     * @param int         $post_id        The ID of the post.
     * @param int|string  $object_id      Optional. The ID of the specific object (e.g., attachment ID) or relevant string. 0 for post-level overrides.
     * @param string      $override_type  The type of override (e.g., 'title', 'meta_description', 'image_alt').
     * @param string      $override_value The new value for the override.
     * @param string      $issue_type     Optional. The specific issue type (e.g., 'missing_meta_description', 'short_title_tag').
     * @param string      $batch_id       Optional. The batch ID for logging.
     * @param string      $row_key        Optional. The row key for logging.
     * @return bool True on success (insert/replace), false on error.
     */
    public static function update_seo_data( $post_id, $object_id, $override_type, $override_value, $issue_type = null, $batch_id = null, $row_key = null ) {
        global $wpdb;

        // Debug logging for short_meta_description issue

        // Ensure RANK_TABLE_NAME is defined
        if ( ! defined('RANK_TABLE_NAME') ) {
             error_log('RANK Plugin Error: RANK_TABLE_NAME constant not defined in Rank_Processor::update_seo_data.');
             return false;
        }
        $table_name = $wpdb->prefix . RANK_TABLE_NAME;
        
        // Check table structure for debugging

        // Ensure $post_id is an integer
        $post_id = intval( $post_id );
        if ( $post_id <= 0 ) {
            error_log("Invalid Post ID for override type $override_type.");
            if ($batch_id) self::add_log_to_transient( $batch_id, "DB Error: Invalid Post ID $post_id for $override_type.", $row_key, 'error' );
            return false;
        }

        // Ensure $object_id is an integer. For non-image types, it should be 0.
        // For image_alt, it should be the attachment_id.
        $object_id = intval( $object_id );
        if ( $override_type === 'image_alt' && $object_id <= 0 ) {
            error_log("Invalid Attachment ID for image_alt.");
            if ($batch_id) self::add_log_to_transient( $batch_id, "DB Error: Invalid Attachment ID $object_id for image_alt.", $row_key, 'error' );
            return false;
        }

        // Sanitize override_type (already mapped from api_issue_type)
        $allowed_types = array( 'meta_description', 'title', 'image_alt' );
        if ( ! in_array( $override_type, $allowed_types, true ) ) {
            error_log("Invalid Override Type: $override_type.");
            if ($batch_id) self::add_log_to_transient( $batch_id, "DB Error: Invalid override type $override_type.", $row_key, 'error' );
            return false;
        }

        // Sanitize override_value based on type
        $sanitized_value = '';
        $value_format = '%s'; // Default to string
        // Make sure override_value is a string before sanitizing
        if (!is_string($override_value)) {
            if (is_array($override_value)) {
                // Convert array to string if needed
                $override_value = isset($override_value[0]) ? (string)$override_value[0] : '';
            } else {
                // Convert any other type to string
                $override_value = (string)$override_value;
            }
        }
        
        // Sanitize the value
        $sanitized_value = sanitize_text_field($override_value);

        $data = array(
            'post_id'        => $post_id,
            'object_id'      => $object_id, // Use the passed $object_id
            'override_type'  => $override_type,
            'issue_type'     => $issue_type ?: $override_type, // Use issue_type if provided, otherwise fallback to override_type
            'override_value' => $sanitized_value,
        );

        $format = array(
            '%d', // post_id
            '%d', // object_id
            '%s', // override_type
            '%s', // issue_type
            $value_format,
        );
        
        // If an override_id exists for this combination, $wpdb->replace will UPDATE it.
        // Otherwise, it will INSERT a new row.
        // This relies on a unique key on (post_id, object_id, override_type).
        // For short_meta_description, log the exact SQL that will be executed
        
        $result = $wpdb->replace( $table_name, $data, $format );
        

        if ( false === $result ) {
            error_log("Failed to update/insert SEO data. WPDB Error: " . $wpdb->last_error);
            return false;
        }
        
        // Log successful database operations for debugging
        
        
        // Clear the health issues cache to ensure fresh data
        if (function_exists('rank_clear_health_issues_cache')) {
            rank_clear_health_issues_cache();
        }
        
        return true;
    }

    /**
     * Attempt to find the attachment ID for a given URL.
     *
     * @param string $url The URL of the attachment.
     * @return int The attachment ID, or 0 if not found.
     */
    public static function get_attachment_id_from_url( $url ) {
        global $wpdb;
        $attachment_id = 0;

        if ( empty( $url ) ) {
            return 0;
        }

        // Try the most reliable WordPress function first
        $attachment_id = attachment_url_to_postid( $url );
        if ( $attachment_id > 0 ) {
            return $attachment_id;
        }

        // Fallback logic if attachment_url_to_postid fails

        $upload_dir_paths = wp_upload_dir();
        if ( !isset($upload_dir_paths['baseurl']) ) {
            return 0;
        }
        $upload_baseurl = $upload_dir_paths['baseurl'];

        // Check if the FULL URL starts with the upload baseurl
        if ( strpos( $url, $upload_baseurl ) === 0 ) {
            // $relative_path is path relative to the uploads directory.
            // Example: $url = 'https://example.com/wp-content/uploads/2023/01/image.jpg'
            // $upload_baseurl = 'https://example.com/wp-content/uploads'
            // str_replace result: '/2023/01/image.jpg'
            // ltrim result: '2023/01/image.jpg'
            $relative_path = ltrim( str_replace( $upload_baseurl, '', $url ), '/' );

            if (empty($relative_path)) { // Should not happen if strpos matched and baseurl is not the full url
                 return 0;
            }

            $filename = basename( $relative_path );
            $directory = dirname( $relative_path );

            if ($directory === '.' || $directory === '') {
                $directory = '';
            } else {
                $directory .= '/';
            }

            $base_relative_path_candidate = $relative_path; 

            $path_parts = pathinfo($filename);
            if (isset($path_parts['filename'])) {
                $fname_no_ext = $path_parts['filename'];
                $fname_no_ext_no_size = preg_replace('/(-\d+x\d+|-scaled|-s\d+)$/i', '', $fname_no_ext);
                $original_extension = isset($path_parts['extension']) ? '.' . $path_parts['extension'] : '';
                
                if ($fname_no_ext_no_size !== $fname_no_ext) {
                    $base_relative_path_candidate = $directory . $fname_no_ext_no_size . $original_extension;
                }
            }

            $sql_like_relative_path = '%' . $wpdb->esc_like( $relative_path );
            $attachment_id = (int) $wpdb->get_var( $wpdb->prepare(
                "SELECT ID FROM {$wpdb->posts} WHERE post_type = 'attachment' AND guid LIKE %s",
                $sql_like_relative_path
            ) );

            if ( ! $attachment_id && $base_relative_path_candidate !== $relative_path && !empty($base_relative_path_candidate) ) {
                $sql_like_base_relative = '%' . $wpdb->esc_like( $base_relative_path_candidate );
                $attachment_id = (int) $wpdb->get_var( $wpdb->prepare(
                    "SELECT ID FROM {$wpdb->posts} WHERE post_type = 'attachment' AND guid LIKE %s",
                    $sql_like_base_relative
                ) );
            }
        } else {
        }
        
        if ($attachment_id > 0) {
        } else {
        }

        return $attachment_id;
    }
    
    /**
     * Enhanced URL to Post ID conversion that handles multilingual sites and edge cases
     * that the standard WordPress url_to_postid() function might miss.
     *
     * @param string $url The URL to convert to a post ID
     * @return int The post ID, or 0 if not found
     */
    public static function enhanced_url_to_postid( $url ) {
        global $wpdb;
        
        // If empty URL, return 0
        if ( empty( $url ) ) {
            return 0;
        }
        
        // Parse the URL to get its components
        $url_parts = parse_url( $url );
        if ( !isset( $url_parts['path'] ) ) {
            return 0;
        }
        
        // Get the path and remove trailing slash
        $path = rtrim( $url_parts['path'], '/' );
        
        // Handle multilingual URLs (e.g., /de/, /fr/, etc.)
        $path_parts = explode( '/', trim( $path, '/' ) );
        
        // Check if the first part is a language code (2 characters)
        $language_prefixes = array( 'de', 'fr', 'es', 'it', 'nl', 'da', 'sv', 'no', 'fi', 'pt', 'ru', 'pl', 'ja', 'zh', 'ko', 'ar' );
        $has_language_prefix = false;
        
        if ( count( $path_parts ) > 0 && strlen( $path_parts[0] ) === 2 && in_array( $path_parts[0], $language_prefixes ) ) {
            $has_language_prefix = true;
            // Remove language prefix for the query
            array_shift( $path_parts );
            $path_without_language = '/' . implode( '/', $path_parts );
        } else {
            $path_without_language = $path;
        }
        
        // Try to find post by path
        $sql = $wpdb->prepare(
            "SELECT ID FROM $wpdb->posts
            WHERE post_status = 'publish'
            AND (post_name = %s OR post_name = %s)
            LIMIT 1",
            trim( $path, '/' ),
            trim( $path_without_language, '/' )
        );
        
        $post_id = $wpdb->get_var( $sql );
        
        if ( $post_id ) {
            return (int) $post_id;
        }
        
        // If not found, try to match against the guid field
        $site_url = site_url();
        $sql = $wpdb->prepare(
            "SELECT ID FROM $wpdb->posts
            WHERE post_status = 'publish'
            AND guid = %s
            LIMIT 1",
            esc_url_raw( $url )
        );
        
        $post_id = $wpdb->get_var( $sql );
        
        if ( $post_id ) {
            return (int) $post_id;
        }
        
        // If still not found, try to match against post meta (for custom permalinks)
        $sql = $wpdb->prepare(
            "SELECT post_id FROM $wpdb->postmeta
            WHERE meta_key = '_wp_page_template'
            AND meta_value LIKE %s
            LIMIT 1",
            '%' . $wpdb->esc_like( basename( $path ) ) . '%'
        );
        
        $post_id = $wpdb->get_var( $sql );
        
        if ( $post_id ) {
            return (int) $post_id;
        }
        
        // Last resort: try to match against the slug in the database
        $slug = basename( $path );
        $sql = $wpdb->prepare(
            "SELECT ID FROM $wpdb->posts
            WHERE post_status = 'publish'
            AND post_name = %s
            LIMIT 1",
            $slug
        );
        
        $post_id = $wpdb->get_var( $sql );
        
        return (int) $post_id;
    }

    /**
     * Adds a log entry to a transient for batch processing feedback.
     *
     * @param string $batch_id   The batch ID to associate the log with.
     * @param string $message    The log message.
     * @param string $row_key    A unique identifier for the specific task.
     * @param string $status     The status of the log entry (success, error, warning, info, critical, suggestion).
     * @return bool              True if the log was added, false otherwise.
     */
    public static function add_log_to_transient($batch_id, $message, $row_key, $status = 'info') {
        if (empty($batch_id) || empty($message)) {
            return false;
        }

        $transient_key = 'rank_batch_log_' . $batch_id;
        $current_logs = get_transient($transient_key);
        
        if (!is_array($current_logs)) {
            $current_logs = array();
        }

        // Add the new log entry
        $current_logs[] = array(
            'time' => current_time('mysql'),
            'message' => $message,
            'row_key' => $row_key,
            'status' => $status
        );

        // Store for 24 hours (86400 seconds)
        return set_transient($transient_key, $current_logs, 86400);
    }
} 